/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.amtools;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Collection;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Mailer;

/**
 * The class <code>MailGroupNotifier</code> sends mails notifications. Gets a identifier of
 * a group, a date and a kind of miss(miss, delay,
 * justified miss or expulsion) and sends a e-mail to all students that match to parameters.
 *  
 *  @author Sergio Santamaría <santamaria.sergio@gmail.com>
 *  @author aalejandro Alex Amigo <aalejando@iespuigcastellar.xeill.net>
 *  */

public class MailGroupNotifier {

	private DatabaseStorage db = new DatabaseStorage();
	private AttendancemanagerApplication app = new AttendancemanagerApplication();
	private static MailGroupNotifier app_mail = new MailGroupNotifier();
	private ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");
	private DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, new Locale(rb.getString("locale")));
	private int test = 0;	//test se utiliza para comprobar si el primer campo es --test, o no.
	private Mailer mail = new Mailer();
	
	/**
	 * @param args	<code>Array</code> of <code>String</code> with user parameters
	 */
	public static void main(String[] args) {
	
		if (app_mail.checkParameters(args)){
			try {
				app_mail.doit(args);
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void doit(String[] args) throws SQLException, ParseException{
		
		String fecha = args[2 + test];
		int err = 0;
		int corr = 0;
		String fallidos = "";
		
		String tipofalta = "";
		if (args[1 + test].compareTo("1") == 0){
			tipofalta = "misscount";
		}
		else if (args[1 + test].compareTo("2") == 0){
			tipofalta = "excusedcount";
		}
		else if (args[1 + test].compareTo("3") == 0){
			tipofalta = "delaycount";
		}
		else if (args[1 + test].compareTo("4") == 0){
			tipofalta = "expulsioncount";
		}

		Date dfecha = df.parse(fecha);
				
		ResultSet resultset = null;
		try {
			resultset = db.studentReport1(Integer.parseInt(args[0 + test]), dfecha, dfecha);
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		while (resultset.next()){
			int n_veces = resultset.getInt(tipofalta);
			String subject = "";
			String bodyText = "";
			
			// Cuando el mail del alumno sea correcto se añade al array de direcciones
			String studentMail = resultset.getString("student.email");
			if (!mail.isValidEmailAddress(studentMail)){
				err++;
				fallidos = (fallidos +resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2") + " -- " + resultset.getString("student.email") + " -- " + bodyText + "\n");
				System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_MAILADDRESS")+ "--> "+ studentMail);
			}
			else{
				String[] toAdresses = { "sergio@localhost" , "root@localhost" , studentMail };
				mail.setTo(toAdresses);
			}
			
			if (n_veces == 1 && args.length == 6+test){
				if (test == 0){
					subject = args[3 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
						bodyText = args[5 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
						mail.setSubject(subject);
						mail.setBodyText(bodyText);
						mail.sendMail();
						corr++;	
				}
				else {
					subject = args[3 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
					bodyText = args[5 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
					mail.setSubject(subject);
					mail.setBodyText(bodyText);
					String str = mail.toString();
					System.out.println(str);
				}
			}
			else if (n_veces == 1 && args.length == 5+test){
				System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERRORMESSAGEPARAMETER_PART1")+" "+ resultset.getString("student.name") + " "+ app.locale.getString("MAILGROUPNOTIFIER_ERRORMESSAGEPARAMETER_PART2"));
				System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_PARAMETERS"));
				System.exit(2);
			}
			else if (n_veces > 1){
				if (test == 0){
					subject = args[3 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
					bodyText = args[4 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
					mail.setSubject(subject);
					mail.setBodyText(bodyText);
					mail.sendMail();
					corr++;	
				}
				else {
					subject = args[3 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
					bodyText = args[4 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
					mail.setSubject(subject);
					mail.setBodyText(bodyText);
					String str = mail.toString();
					System.out.println(str);
				}
			}
		}
		if (test == 0){
			System.out.print(app.locale.getString("MAILGROUPNOTIFIER_MAILINFO_SEND") + corr + app.locale.getString("MAILGROUPNOTIFIER_MAILINFO_ERROR") + err);
			if (err > 0){
				System.out.println(" :\n" + fallidos);
				System.exit(1);
			}
		}
	}
	
	/**
     * Este método comprueba si los argumentos introducidos por el usuario,
     * son correctos. Comprueba la cantidad de argumentos y el tipo de cada uno
     * de ellos.
     *      
     * @param    args    Una <code>Array</code> de <code>String</code> con los argumentos
     * introducidos por el usuario.
     * 
     * @return cierto si los parámetros con correctos.
     */
	public boolean checkParameters(String[] args){
		
		//Comprueba si el primer parámetro es --test
		if((args.length > 0) && (args[0].compareTo("--test") == 0)){
			test = 1;
		}
		
		//Comprueba el número de argumentos introducidos por el usuario
		if (args.length < 5 + test){
			if ((args.length == 1) && (args[0].compareTo("--help") == 0)){
				System.out.println(app.locale.getString("MAILGROUPNOTIFIER_HELP_INSTRUCTIONS") + 
						app.locale.getString("MAILGROUPNOTIFIER_HELP_OPTIONAL") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_TEST") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_LISTGROUP") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_DATE") + df.format(new Date()) + "\n\n" +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_TYPESOFFAULTSA") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_TYPESOFFAULTSB") +
						app.locale.getString("MAILGROUPNOTIFIER_HELP_PARAMETERS") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSN") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSA") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSB") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSC") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSD") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSF") +
						app.locale.getString("MAILGROUPNOTIFIER_HELP_EXAMPLE"));
			}
			else if((args.length == 1) && (args[0].compareTo("--list-groups") == 0)){
				Collection<Group> grupos = db.allGroups();
				System.out.println(app.locale.getString("SMSGROUPNOTIFIER_GROUPINFO"));
				for (Group gr : grupos) {
					System.out.println(gr.getId() + " -- " + gr.getName() + " -- " + gr.getComment());
				}
			}		
			else{
				System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_PARAMETERS"));
				System.exit(2);
			}
			return false;
		}
		else{
			//Comprueba si el identificador del grupo introducido por el usuario es un entero.
			try{
				int idgroup = Integer.parseInt(args[0 + test]);
			}catch (Exception e){
				System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_IDGROUP"));
				System.exit(3);
				return false;
			}
			
			//Comprueba si el tipo de falta introducido por el usuario es un entero entre 1 y 4.
			try{
				int tipo_falta = Integer.parseInt(args[1 + test]);
				if (tipo_falta < 1 || tipo_falta>4){
					System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_MISS"));
					System.exit(4);
					return false;
				}
			}catch (Exception e){
				System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_MISS"));
				System.exit(4);
				return false;
			}
			
			//Comprueba si el tipo de fecha introducido es correcto
			try{
				Date dfecha = df.parse(args[2 + test]);
			}catch (Exception e){
				Date dfecha = new Date();
				System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_DATE") + df.format(dfecha));
				System.exit(5);
				return false;
			}
			
			//Comprueba que el mensaje de texto sea correcto
			if (args.length == 6 + test){
				if (checkBodyText(args[3 + test]) && checkBodyText(args[4 + test]) && checkBodyText(args[5 + test])) return true;
				else{
					System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_TEXT"));
					System.exit(6);
					return false;
				}
			}
			else{
				if (checkBodyText(args[3 + test]) && checkBodyText(args[4 + test ])) return true;
				else{
					System.out.println(app.locale.getString("MAILGROUPNOTIFIER_ERROR_TEXT"));
					System.exit(6);
					return false;
				}
			}
		}
	}
	
	/**
     * Este método comprueba la cadena del e-mail. Dada una cadena
     * introducida por el usuario, comprueba si es correcta. Comprueba
     * el tipo de parámetros opciones del mensaje. 
     *
     * @param    bodyText    Una <code>String</code> con el texto a enviar.
     * 
     * @return cierto si el texto a enviar es correcto.
     */
	public boolean checkBodyText(String bodyText){
		String aux = "";
		
		for (int i = 0; i < bodyText.length(); i++){
			if (aux.compareTo("%") == 0){
				if (bodyText.charAt(i) == '%' || (bodyText.charAt(i) != 'n' && bodyText.charAt(i) != 'a' && bodyText.charAt(i) != 'b' && bodyText.charAt(i) != 'c' && bodyText.charAt(i) != 'd' && bodyText.charAt(i) != 'f')){
					return false;
				}
				else{
					aux += bodyText.charAt(i);
				}
			}

			else if (bodyText.charAt(i) == '%') aux = "%";
		}
		return true;
	}
}
