/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.amtools;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Collection;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Logger;
import org.iespuigcastellar.attendancemanager.core.SMS;

/**
 * Esta clase envía notificaciones de faltas vía SMS. Dado el identificador
 * de un grupo, una fecha determinada ,un tipo de falta (falta, retraso,
 * justificación o expulsión) y un patrón de mensaje, envía un SMS al teléfono
 * de contacto de aquellos alumnos que pertenezcan al grupo indicado y hayan cometido
 * alguna falta del tipo indicado, el día dado por el usuario.
 * 
 * @author aalejandro Alex Amigo <aalejando@iespuigcastellar.xeill.net>
 *
 */
public class SMSGroupNotifier {

	private DatabaseStorage db = new DatabaseStorage();
	private AttendancemanagerApplication app = new AttendancemanagerApplication();
	private static SMSGroupNotifier app_sms = new SMSGroupNotifier();
	private ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");
	private DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, new Locale(rb.getString("locale")));

	private int test = 0;	//test, se utiliza para comprobar si el primer campo es --test, o no, 0 no se ha introducido.

	private SMS sms = new SMS();

	/**
	 * @param args	Una <code>Array</code> de <code>String</code> con los argumentos
     * introducidos por el usuario.
	 */
	public static void main(String[] args) {
	
		if (app_sms.checkParameters(args)){
			try {
				app_sms.doit(args);
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	

	public void doit(String[] args) throws SQLException, ParseException{		

		String fecha = args[2 + test];
		int err = 0;
		int corr = 0;
		String fallidos = "";
		
		String tipofalta = "";
		if (args[1 + test].compareTo("1") == 0){
			tipofalta = "misscount";
		}
		else if (args[1 + test].compareTo("2") == 0){
			tipofalta = "excusedcount";
		}
		else if (args[1 + test].compareTo("3") == 0){
			tipofalta = "delaycount";
		}
		else if (args[1 + test].compareTo("4") == 0){
			tipofalta = "expulsioncount";
		}

		Date dfecha = df.parse(fecha);
				
		ResultSet resultset = null;
		try {
			resultset = db.studentReport1(Integer.parseInt(args[0 + test]), dfecha, dfecha);
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		while (resultset.next()){
			int n_veces = resultset.getInt(tipofalta);
			String text = "";
			if ((n_veces == 1) && (args.length == 5 + test)){
				text = args[4 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
			}
			else {
				text = args[3 + test].replace("%n", resultset.getString("student.name")).replace("%a", resultset.getString("student.surname1")).replace("%b", resultset.getString("student.surname2")).replace("%c",resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2")).replace("%d",fecha).replace("%f","" + resultset.getInt(tipofalta));
			}
			if (n_veces > 0){
				if (test == 0){
					if(checkLengthSmsText(text)) {
						if ((resultset.getString("student.smsphone") == null) || (!sms.sendSMS(resultset.getString("student.smsphone"), text))){
							err++;
							fallidos = (fallidos +resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2") + " -- " + resultset.getString("student.smsphone") + " -- " + text + "\n");  
						}
						else corr++;
					}
					else{
						err++;
						fallidos = (fallidos +resultset.getString("student.name") + " " + resultset.getString("student.surname1") + " " + resultset.getString("student.surname2") + " -- " + resultset.getString("student.smsphone") + " -- " + text + "\n");
					}
				}
				else {
					System.out.println(app.locale.getString("SMSGROUPNOTIFIER_INFO_PHONE") + resultset.getString("student.smsphone") + app.locale.getString("SMSGROUPNOTIFIER_INFO_MESSAGE") + text);
				}
			}
		}
		if (test == 0){

			System.out.print(app.locale.getString("SMSGROUPNOTIFIER_SMSINFO_SEND") + corr + app.locale.getString("SMSGROUPNOTIFIER_SMSINFO_ERROR") + err);

			if (err > 0){
				System.out.println(" :\n" + fallidos);
				System.exit(1);
			}
		}
	}
	
	/**
     * Este método comprueba si los argumentos introducidos por el usuario,
     * son correctos. Comprueba la cantidad de argumentos y el tipo de cada uno
     * de ellos.
     *      
     * @param    args    Una <code>Array</code> de <code>String</code> con los argumentos
     * introducidos por el usuario.
     * 
     * @return cierto si los parámetros con correctos.
     */
	public boolean checkParameters(String[] args){
		
		//Comprueba si el primer parámetro es --test
		if((args.length > 0) && (args[0].compareTo("--test") == 0)){
			test = 1;
		}
		
		//Comprueba el número de argumentos introducidos por el usuario
		if (args.length < 4 + test){
			if ((args.length == 1) && (args[0].compareTo("--help") == 0)){

				System.out.println(app.locale.getString("SMSGROUPNOTIFIER_HELP_INSTRUCTIONS") + 
						app.locale.getString("SMSGROUPNOTIFIER_HELP_OPTIONAL") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_TEST") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_LISTGROUP") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_DATE") + df.format(new Date()) + "\n\n" +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_TYPESOFFAULTSA") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_TYPESOFFAULTSB") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERS") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSN") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSA") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSB") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSC") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSD") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_PARAMETERSF") +
						app.locale.getString("SMSGROUPNOTIFIER_HELP_EXAMPLE"));

			}
			else if((args.length == 1) && (args[0].compareTo("--list-groups") == 0)){
				Collection<Group> grupos = db.allGroups();
				System.out.println(app.locale.getString("SMSGROUPNOTIFIER_GROUPINFO"));
				for (Group gr : grupos) {
					System.out.println(gr.getId() + " -- " + gr.getName() + " -- " + gr.getComment());
				}
			}		
			else{
				System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_PARAMETERS"));
				System.exit(2);
			}
			return false;
		}
		else{
			//Comprueba si el identificador del grupo introducido por el usuario es un entero.
			try{
				int idgroup = Integer.parseInt(args[0 + test]);
			}catch (Exception e){
				System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_IDGROUP"));
				System.exit(3);
				return false;
			}
			
			//Comprueba si el tipo de falta introducido por el usuario es un entero entre 1 y 4.
			try{
				int tipo_falta = Integer.parseInt(args[1 + test]);
				if (tipo_falta < 1 || tipo_falta>4){
					System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_MISS"));
					System.exit(4);
					return false;
				}
			}catch (Exception e){
				System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_MISS"));
				System.exit(4);
				return false;
			}
			
			//Comprueba si el tipo de fecha introducido es correcto
			try{
				Date dfecha = df.parse(args[2 + test]);
			}catch (Exception e){
				Date dfecha = new Date();
				System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_DATE") + df.format(dfecha));
				System.exit(5);
				return false;
			}
			
			//Comprueba que el mensaje de texto sea correcto
			if (args.length == 5 + test){
				if (checkSmsText(args[3 + test]) && checkSmsText(args[4 + test])) return true;
				else{
					System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_SMS"));
					System.exit(6);
					return false;
				}
			}
			else{
				if (checkSmsText(args[3 + test ])) return true;
				else{
					System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_SMS"));
					System.exit(6);
					return false;
				}
			}
		}
	}
	
	/**
     * Este método comprueba la cadena del SMS. Dada una cadena
     * introducida por el usuario, comprueba si es correcta. Comprueba
     * el tipo de parámetros opciones del mensaje. 
     *
     * @param    text    Una <code>String</code> con el texto a enviar.
     * 
     * @return cierto si el texto a enviar es correcto.
     */
	public boolean checkSmsText(String text){
		String aux = "";
		
		for (int i = 0; i < text.length(); i++){
			if (aux.compareTo("%") == 0){
				if (text.charAt(i) == '%' || (text.charAt(i) != 'n' && text.charAt(i) != 'a' && text.charAt(i) != 'b' && text.charAt(i) != 'c' && text.charAt(i) != 'd' && text.charAt(i) != 'f')){
					return false;
				}
				else{
					aux += text.charAt(i);
				}
			}

			else if (text.charAt(i) == '%') aux = "%";
		}
		return true;
	}
	
	/**
     * Este método comprueba la longitud de la cadena del SMS. Dada una cadena
     * introducida por el usuario, comprueba si no supera una longitud determinada
	 * por el programa. Este valor depende el número de mensajes concatenados 
	 * permitidos para enviar.
	 *
     * @param    text    Una <code>String</code> con el texto a enviar.
     * 
     * @return cierto si la longitud del texto a enviear es correcta.
     */
	 public boolean checkLengthSmsText(String text){
		 int sms_n = Integer.parseInt(rb.getString("sms_max_number"));
		 if (sms_n == 1){
			 if (text.length() > 70){
				 Logger.log(app.locale.getString("SMSGROUPNOTIFIER_LOG_LONGSMS") + text);
				 System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_LONGSMS" + text));
				 return false;
			 }
		 }
		
		 else if (text.length() > (63 * sms_n)){
			 Logger.log(app.locale.getString("SMSGROUPNOTIFIER_LOG_LONGSMS") + text);
			 System.out.println(app.locale.getString("SMSGROUPNOTIFIER_ERROR_LONGSMS") + text);
			 return false;
		 }
		 
		 return true;
	 }
}
