/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package org.iespuigcastellar.attendancemanager.core;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;

import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Font;
import com.itextpdf.text.Image;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;



/**
 * Esta clase permite generar informes en formato PDF. Los informes se escriben
 * en un ByteArrayOutputStream. Cada tipo de informe se genera en función
 * de un id de grupo y de una fecha de inicio y otra de final, creando un informe
 * entre ambas fechas y el grupo dado.
 * 
 * @author dperez <dperez@iespuigcastellar.xeill.net>
 */

public class CreatePDF  {


	private   ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");
	private DatabaseStorage database = new DatabaseStorage();
	private DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, new Locale(rb.getString("locale")));

	
	public ByteArrayOutputStream createReport1(int idgroup, Date dfecha1, Date dfecha2) throws SQLException, DocumentException, MalformedURLException, IOException, ParseException{
		
//		DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, new Locale(rb.getString("locale")));

	
		
		Document document = new Document();
		
		// Create a new ByteArrayOutputStream to generate the document on-fly and return it for mailer class use on
		
		ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		
		PdfWriter.getInstance(document, byteArrayOutputStream);
		
		document.open();
		
		Image img1 = Image.getInstance(rb.getString("report_logo_file"));
		document.add(img1);
		
		Chunk c1 = new Chunk("Informe de faltas de asistencia por grupo \n",new Font(Font.FontFamily.HELVETICA, 14));
		Chunk c2 = new Chunk("Grupo: "+database.getGroupName(idgroup),new Font(Font.FontFamily.HELVETICA, 14));
		Teacher teacher=database.getGroupTeacher(idgroup);
		Chunk c3 = new Chunk("Tutor: "+teacher.getFullname(),new Font(Font.FontFamily.HELVETICA, 14));
		Chunk c4 = new Chunk(""+df.format(dfecha1)+ " - "+df.format(dfecha2)+"",new Font(Font.FontFamily.HELVETICA, 14));
		
		Paragraph p1=new Paragraph(c1);
		Paragraph p2=new Paragraph(c2);
		Paragraph p3=new Paragraph(c3);
		Paragraph p4=new Paragraph(c4);
		c1.setUnderline(0.2f, -2f);
		
		document.add(p1);
		document.add(p2);
		document.add(p3);
		document.add(p4);
		
		PdfPTable table = new PdfPTable(7);
		
		table.setSpacingBefore(20);
		
		PdfPCell cell1=new PdfPCell(new Paragraph(new Chunk("1er Apellido",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell2= new PdfPCell(new Paragraph(new Chunk("2º Apellido",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell3= new PdfPCell(new Paragraph(new Chunk("Nombre",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell4= new PdfPCell(new Paragraph(new Chunk("Faltas",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell5= new PdfPCell(new Paragraph(new Chunk("Retrasos",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell6= new PdfPCell(new Paragraph(new Chunk("Expulsiones",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell7= new PdfPCell(new Paragraph(new Chunk("Justificadas",new Font(Font.FontFamily.HELVETICA,10))));
		
		table.setWidthPercentage(100);
		
		cell1.setGrayFill(0.7F);
		cell2.setGrayFill(0.7F);
		cell3.setGrayFill(0.7F);
		cell4.setGrayFill(0.7F);
		cell5.setGrayFill(0.7F);
		cell6.setGrayFill(0.7F);
		cell7.setGrayFill(0.7F);
		
		table.addCell(cell1);
		table.addCell(cell2);
		table.addCell(cell3);
		table.addCell(cell4);
		table.addCell(cell5);
		table.addCell(cell6);
		table.addCell(cell7);
		
		ResultSet result = database.studentReport1(idgroup,dfecha1, dfecha2);
	
			while (result.next()) {
				String nombre= result.getString("name");
				String apellido1= result.getString("surname1");
				String apellido2= result.getString("surname2");
				int miss= result.getInt("misscount");
				int excused= result.getInt("excusedcount");
				int delay= result.getInt("delaycount");
				int expulsion= result.getInt("expulsioncount");
				
				
				PdfPCell aux1= new PdfPCell(new Paragraph(new Chunk(apellido1,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux2= new PdfPCell(new Paragraph(new Chunk(apellido2,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux3= new PdfPCell(new Paragraph(new Chunk(nombre,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux4= new PdfPCell(new Paragraph(new Chunk(""+miss,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux5= new PdfPCell(new Paragraph(new Chunk(""+delay,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux6= new PdfPCell(new Paragraph(new Chunk(""+expulsion,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux7= new PdfPCell(new Paragraph(new Chunk(""+excused,new Font(Font.FontFamily.HELVETICA,8))));
				
				table.addCell(aux1);
				table.addCell(aux2);
				table.addCell(aux3);
				table.addCell(aux4);
				table.addCell(aux5);
				table.addCell(aux6);
				table.addCell(aux7);
			}
			document.add(table);
		document.close();
		System.out.println("Informe Generado!!!");
		
		return byteArrayOutputStream;
		
	}


	public ByteArrayOutputStream createReport2(int idgroup, Date dfecha1, Date dfecha2) throws SQLException, DocumentException, MalformedURLException, IOException, ParseException{
		
		Document document = new Document();
				
		ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		PdfWriter.getInstance(document, byteArrayOutputStream);

		document.open();
		
		Image img1 = Image.getInstance(rb.getString("report_logo_file"));
		document.add(img1);
		
		
		Chunk c1 = new Chunk("Informe de faltas de asistencia por asignatura",new Font(Font.FontFamily.HELVETICA, 14));
		Chunk c2 = new Chunk("Grupo: "+database.getGroupName(idgroup),new Font(Font.FontFamily.HELVETICA, 14));
		Teacher teacher=database.getGroupTeacher(idgroup);
		Chunk c3 = new Chunk("Tutor: "+teacher.getFullname(), new Font(Font.FontFamily.HELVETICA, 14));
		Chunk c4 = new Chunk(""+df.format(dfecha1)+ " - "+df.format(dfecha2)+"",new Font(Font.FontFamily.HELVETICA, 14));
		
		Paragraph p1=new Paragraph(c1);
		Paragraph p2=new Paragraph(c2);
		Paragraph p3=new Paragraph(c3);
		Paragraph p4=new Paragraph(c4);
		c1.setUnderline(0.2f, -2f);
		
				
		document.add(p1);
		document.add(p2);
		document.add(p3);
		document.add(p4);
		PdfPTable table = new PdfPTable(8);

		table.setSpacingBefore(20);
		
		//Modificar la fuente y tamaño de la cabecera para que no se descuadre!!!
		//PdfPCell cell1= new PdfPCell(new Paragraph("1er Apellido"));
		PdfPCell cell1=new PdfPCell(new Paragraph(new Chunk("1er Apellido",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell2= new PdfPCell(new Paragraph(new Chunk("2º Apellido",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell3= new PdfPCell(new Paragraph(new Chunk("Nombre",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell4= new PdfPCell(new Paragraph(new Chunk("Asignatura",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell5= new PdfPCell(new Paragraph(new Chunk("Faltas",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell6= new PdfPCell(new Paragraph(new Chunk("Retrasos",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell7= new PdfPCell(new Paragraph(new Chunk("Expulsiones",new Font(Font.FontFamily.HELVETICA,10))));
		PdfPCell cell8= new PdfPCell(new Paragraph(new Chunk("Justificadas",new Font(Font.FontFamily.HELVETICA,10))));
		
		
		table.setWidthPercentage(100);
		
		
		cell1.setGrayFill(0.7F);
		cell2.setGrayFill(0.7F);
		cell3.setGrayFill(0.7F);
		cell4.setGrayFill(0.7F);
		cell5.setGrayFill(0.7F);
		cell6.setGrayFill(0.7F);
		cell7.setGrayFill(0.7F);
		cell8.setGrayFill(0.7F);
		
		table.addCell(cell1);
		table.addCell(cell2);
		table.addCell(cell3);
		table.addCell(cell4);
		table.addCell(cell5);
		table.addCell(cell6);
		table.addCell(cell7);
		table.addCell(cell8);
		
		ResultSet result = database.studentReport2(idgroup,dfecha1, dfecha2);
		
			while (result.next()) {
				
				String nombre= result.getString("name");
				String apellido1= result.getString("surname1");
				String apellido2= result.getString("surname2");
				String materia= result.getString("subject");
				int miss= result.getInt("misscount");
				int excused= result.getInt("excusedcount");
				int delay= result.getInt("delaycount");
				int expulsion= result.getInt("expulsioncount");
				
				
				PdfPCell aux1= new PdfPCell(new Paragraph(new Chunk(apellido1,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux2= new PdfPCell(new Paragraph(new Chunk(apellido2,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux3= new PdfPCell(new Paragraph(new Chunk(nombre,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux4= new PdfPCell(new Paragraph(new Chunk(materia,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux5= new PdfPCell(new Paragraph(new Chunk(""+miss,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux6= new PdfPCell(new Paragraph(new Chunk(""+delay,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux7= new PdfPCell(new Paragraph(new Chunk(""+expulsion,new Font(Font.FontFamily.HELVETICA,8))));
				PdfPCell aux8= new PdfPCell(new Paragraph(new Chunk(""+excused,new Font(Font.FontFamily.HELVETICA,8))));
				
				table.addCell(aux1);
				table.addCell(aux2);
				table.addCell(aux3);
				table.addCell(aux4);
				table.addCell(aux5);
				table.addCell(aux6);
				table.addCell(aux7);
				table.addCell(aux8);

			}
			document.add(table);
		document.close();
		System.out.println("Informe Generado!!!");
		
		return byteArrayOutputStream;
	}
	public ByteArrayOutputStream createReport3(int idgroup, Date dfecha1) throws DocumentException, MalformedURLException, IOException, ParseException{

	
		
		Document document = new Document(PageSize.A4.rotate());
		
		// Create a new ByteArrayOutputStream to generate the document on-fly and return it for mailer class use on
		
		ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		
		PdfWriter.getInstance(document, byteArrayOutputStream);
		
		
		document.open();
		
		
		
		Image img1 = Image.getInstance(rb.getString("report_logo_file"));
		System.out.println("Logo: " + rb.getString("report_logo_file"));
		document.add(img1);
		
		Chunk c1 = new Chunk("Informe de faltas de asistencia semanal\n",new Font(Font.FontFamily.HELVETICA, 14));
		Chunk c2 = new Chunk("Grupo: "+database.getGroupName(idgroup),new Font(Font.FontFamily.HELVETICA, 14));
		Teacher teacher=database.getGroupTeacher(idgroup);
		Chunk c3 = new Chunk("Tutor: "+teacher.getFullname(),new Font(Font.FontFamily.HELVETICA, 14));
		Chunk c4 = new Chunk("Semana del: "+df.format(dfecha1),new Font(Font.FontFamily.HELVETICA, 14));
		
		Paragraph p1=new Paragraph(c1);
		Paragraph p2=new Paragraph(c2);
		Paragraph p3=new Paragraph(c3);
		Paragraph p4=new Paragraph(c4);
		c1.setUnderline(0.2f, -2f);
		
		document.add(p1);
		document.add(p2);
		document.add(p3);
		document.add(p4);
		
		Calendar calendar = Calendar.getInstance();
	    calendar.setTime(dfecha1);
	    Vector colecciones[] = new Vector[5];
	    //Coger classblocks de cada dia
	    Date fechas[] = new Date[5];
	    for(int k=0;k<5;k++){
	    	fechas[k]=dfecha1;
	        System.out.println(calendar.getTime().toString());
	    	colecciones[k]=(Vector<Classblock>) database.getClassblocksOnDate(idgroup, dfecha1);
	    	calendar.roll(calendar.DATE, true);	    	
	    	if (calendar.getTime().before(dfecha1)) calendar.roll(Calendar.MONTH, true);
	    	dfecha1 = calendar.getTime();
	    }
	    System.out.println("Array de vectores llenos");
	    int size=0;
	    for(int i=0; i<colecciones.length;i++){
	    	size=size+colecciones[i].size();
	    }
	    System.out.println("Tamaño tabla calculado");
	    PdfPTable table = new PdfPTable(size+1);
	    
		PdfPCell blanco=new PdfPCell(new Paragraph(new Chunk("",new Font(Font.FontFamily.HELVETICA,10))));
		blanco.setGrayFill(0.7F);
		table.addCell(blanco);
	    
	    String days[]= new String[]{"Lunes","Martes","Miercoles","Jueves","Viernes"};
	    
		for(int i=0;i<colecciones.length;i++){
				PdfPCell aux=new PdfPCell(new Paragraph(new Chunk(days[i],new Font(Font.FontFamily.HELVETICA,10))));
				aux.setGrayFill(0.7F);
				aux.setColspan(colecciones[i].size());
				table.addCell(aux);
		}
		PdfPCell aux1=new PdfPCell(new Paragraph(new Chunk("Alumno",new Font(Font.FontFamily.HELVETICA,10))));
		aux1.setGrayFill(0.7F);
		table.addCell(aux1);
	    //Imprimir cabecera

		DateFormat df2 = DateFormat.getTimeInstance(DateFormat.SHORT, new Locale(rb.getString("locale")));
		
		for(int i=0;i<colecciones.length;i++){
			for(int j=0;j<colecciones[i].size();j++){
				Classblock cb=(Classblock) colecciones[i].elementAt(j);
				PdfPCell aux=new PdfPCell(new Paragraph(new Chunk(df2.format(cb.getStart())+" "+df2.format(cb.getEnd()),new Font(Font.FontFamily.HELVETICA,10))));
				aux.setGrayFill(0.7F);
				table.addCell(aux);
			}
		}
		
		int widths[]= new int[size+1];
		widths[0]=40;
		for(int i=1;i<size+1;i++){
			widths[i]=30;
		}
		table.setWidths(widths);

		
		System.out.println("Cabecera imprimida");
		Vector<Student> students=(Vector<Student>) database.getStudentsCollection(idgroup);
		System.out.println("Vector de estudiantes lleno "+students.size());
		//Comprobar faltas y añadir a la tabla
		for(int i=0;i<students.size();i++){
			Student s=students.elementAt(i);
			PdfPCell aux=new PdfPCell(new Paragraph(new Chunk(s.getSurname1()+" "+s.getSurname2()+" ,"+s.getName(),new Font(Font.FontFamily.HELVETICA,10))));
			table.addCell(aux);
			System.out.println("Estudiante: "+i);
			for(int k=0;k<colecciones.length;k++){
				for(int j=0;j<colecciones[k].size();j++){
					Date date=fechas[k];
					Classblock cb=(Classblock) colecciones[k].elementAt(j);
					String text="";
					if(database.existsMiss(1, date, cb.getID(), s.getID())) {
						text=text + " F";
					}	
					if(database.existsMiss(2, date, cb.getID(), s.getID())) {
						text=text + " R";
					}
					if(database.existsMiss(3, date, cb.getID(), s.getID())) {
						text=text + " J";
					}
					if(database.existsMiss(4, date, cb.getID(), s.getID())) {
						text=text + " E";
					}
					PdfPCell aux2=new PdfPCell(new Paragraph(new Chunk(text,new Font(Font.FontFamily.HELVETICA,10))));
					table.addCell(aux2);
				}
				
			}
		}

		table.setSpacingBefore(20);
		
		table.setWidthPercentage(100);
		document.add(table);
		document.close();
		
		return byteArrayOutputStream;
	}
	
}