/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.security.MessageDigest;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.Vector;

import javax.xml.bind.annotation.adapters.HexBinaryAdapter;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;

import com.vaadin.data.Item;
import com.vaadin.data.util.QueryContainer;
import com.vaadin.ui.Table;

/**
 * Provides access to a DBMS, actually MySQL. A DatabaseStorage maintains
 * a connection with the DBMS allowing different operations. Once the method
 * <code>close()<code> is executed the connection is closed.
 * 
 * Also provides property names as constants and hash generator for password
 * strings.
 * 
 * @author vcarceler <vcarceler@iespuigcastellar.xeill.net>
 * @author fjunyent <fjunyent@iespuigcastellar.xeill.net>
 * @author dperez <dperez@iespuigcastellar.xeill.net>
 */
public class DatabaseStorage {

	private Connection con;
	private Statement statement;
	private String database;		// We need to do database+".group" in some queries
	private AttendancemanagerApplication app;
	
	private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
	private SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm");
	
	private MessageDigest md;
	private HexBinaryAdapter hbinary = new HexBinaryAdapter();
	
	// Define property ID strings to refer table columns
	public static final String IDGROUP_PROPERTY = "idgroup";
	public static final String NAME_PROPERTY = "name";
	public static final String COMMENT_PROPERTY = "comment";
	public static final String STUDENTS_PROPERTY = "students";	// Used by GroupsQueryContainer
	public static final String IDSTUDENT_PROPERTY = "idstudent";
	public static final String IDTEACHER_PROPERTY ="idteacher";
	public static final String SURNAME1_PROPERTY = "surname1";
	public static final String SURNAME2_PROPERTY = "surname2";
	public static final String EMAIL_PROPERTY = "email";
	public static final String SMSPHONE_PROPERTY = "smsphone";
	public static final String LOGIN_PROPERTY = "login";
	public static final String FULLNAME_PROPERTY = "fullname";
	public static final String ADMIN_PROPERTY = "admin";
	public static final String PASSWORD_PROPERTY="password";
	public static final String START_PROPERTY="start";
	public static final String END_PROPERTY="end";
	public static final String TEACHER_IDTEACHER_PROPERTY="teacher_idteacher";
	public static final String MANAGERTEACHER_PROPERTY="tutor";
	public static final String IDCLASSBLOCK_PROPERTY="idclassblock";
	public static final String CLASSBLOCK_IDCLASSBLOCK_PROPERTY="classblock_idclassblock";
	public static final String SUBJECT_IDSUBJECT_PROPERTY="subject_idsubject";
	public static final String GROUP_IDGROUP_PROPERTY="group_idgroup";
	public static final String TERM_IDTERM_PROPERTY="term_idterm";
	public static final String TEACHERNAME_PROPERTY="teachername";
	public static final String TERMNAME_PROPERTY="termname";
	public static final String GROUPNAME_PROPERTY="groupname";
	public static final String SUBJECTNAME_PROPERTY="subjectname";
	public static final String DAYOFWEEK_PROPERTY="day_of_week";
	public static final String DAYOFWEEKSTRING_PROPERTY="dayofweekstring";
	public static final String DATE_PROPERTY="date";
	public static final String TIME_PROPERTY="time";
	public static final String STUDENT_IDSTUDENT_PROPERTY="student_idstudent";
	
	public DatabaseStorage() {
		try {
			
			ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");
			database = rb.getString("mysql_database");
			
			Class.forName("com.mysql.jdbc.Driver");
			String url = "jdbc:mysql://" + rb.getString("mysql_server") + ":" + rb.getString("mysql_port") + "/" + database;
			con = DriverManager.getConnection(url, rb.getString("mysql_user"), rb.getString("mysql_password"));
			statement = con.createStatement();
			Logger.log("DatabaseStorage() DB connection established");
			
			md = MessageDigest.getInstance("SHA-1"); 
			
		} catch(Exception e) {
			Logger.log("DatabaseStorage.DatabaseStorage()");
			System.err.println(e);
		}

	}
	
	/** 
	 * Closes the connection with database. Once closed the connection
	 * can't be reestablished, make a new instance if you want another
	 * connection.
	 */
	public void close() {
		try {
			con.close();
			Logger.log("DatabaseStorage.close() DB connection closed");
		} catch (SQLException e) {
			System.err.println("DatabaseStorage.close()");
			System.err.println(e);
			e.printStackTrace();
		}
	}

	/** Looks for the login/password pair in Teacher table, returns Teacher object if
	 * it's a real user or null if login fails.
	 * 
	 * @param login Login string to look for
	 * @param password Password string to look for
	 * @return Teacher corresponding to login/password pair or null 
	 */
	public Teacher loginUser(String login, String password) {
		
		Teacher user = null;
		
		try {
			
			String passwordhash = hbinary.marshal(md.digest(password.getBytes())).toLowerCase();
			
			ResultSet result = statement.executeQuery("SELECT idteacher, fullname, email, comment, admin FROM teacher WHERE teacher.login = '" + login + "' AND teacher.password = '" + passwordhash + "';");
			if(result.next()) {
				user = new Teacher(result.getInt("idteacher"), login, passwordhash, result.getString("fullname"), result.getString("email"), result.getString("comment"), result.getBoolean("admin"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return user;
	}
	
	/** 
	 *	Query DBMS for login string existence
	 *
	 * @return true if login string exists false otherwise
	 */
	// QUÉ ES ESTO, QUIÉN LO UTILIZA ???
	public boolean loginExists(String login) {
		boolean exists = false;
		
		try {
			ResultSet result = statement.executeQuery("SELECT login FROM teacher WHERE teacher.login = '" + login + "';");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
	}
	
	/** Returns true if given 'login' exists and is different that edited teacher , false otherwise */
	// QUÉ ES ESTO, QUIÉN LO UTILIZA ???
	public boolean loginExists(String login,Integer idTeacher) {
		boolean exists = false;
		
		try {
			System.out.println(idTeacher);
			ResultSet result = statement.executeQuery("SELECT login FROM teacher WHERE teacher.login = '" + login + "' AND idteacher <> "+ idTeacher + ";");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
	}

	/**
	 * Returns a collection with all the students of a group.
	 * 
	 * @param idgroup group to get students from
	 * @return Collection with all the students of the group, may be empty
	 */
	public Collection<Student> studentsOfGroup(int idgroup) {
		Vector<Student> vector = new Vector<Student>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT idstudent, surname1, surname2, name, email, smsphone FROM student, group_has_student WHERE group_has_student.group_idgroup = " + idgroup + " AND group_has_student.student_idstudent = student.idstudent ORDER BY surname1, surname2, name;");
			while(result.next()) {
				vector.add(new Student(result.getInt("idstudent"), result.getString("surname1"), result.getString("surname2"), result.getString("name"), result.getString("email"),result.getString("smsphone")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}
	
	/**
	 * Adds group to the database and returns the assigned ID or 0 if there is some
	 * problem. A group gets one ID when stored in database for the first time.
	 * 
	 * @return ID assigned to group in DBMS or 0 if not stored at all
	 */
	public int addGroup(Group group) {
		int id = 0;
		Integer idteacher=null;
		
		try {

			if (group.getIdteacher()!=0){idteacher=group.getIdteacher();}
			statement.execute("INSERT INTO " + database + ".group VALUES('0', '" + group.getName() + "', '" + group.getComment() + "'," + idteacher + ");");

			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
		} catch (SQLException e) {
			Logger.log("DatabaseStorage.addGroup(" + group + ")");
			e.printStackTrace();
		}
		return id;
	}
	
    /**
     * Updates group in DBMS.
     * 
     * @param group to update in DBMS
     * @return number of updated records
     */
    public int updateGroup(Group group) {
    	Integer id=group.getId();
    	String name=group.getName();
    	String comment=group.getComment();

    	Integer idteacher=null;
    	if (group.getIdteacher()!=0){idteacher=group.getIdteacher();}
        String sql="UPDATE " + database + ".group SET name='" + name + "' , comment='" + comment + "', teacher_idteacher=" + idteacher + " WHERE idgroup=" + id + ";";

        int res=0;
        
        try{
        	res = statement.executeUpdate(sql);
        } catch (SQLException e) {
        	e.printStackTrace();
        }

        return res;
    }
    
    /**
     * Checks if a group has students or is empty.
     * 
     * @param g Group to check if has any student
     * @return true if the group is not empty
     */
    public boolean groupHasStudents(Group g){
		boolean exists = false;
		
		try {
			ResultSet result = statement.executeQuery("SELECT group_idgroup FROM group_has_student WHERE  group_idgroup=" + g.getId() + ";");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;

    }
    
    /**
     * Check if a group has associated classblocks.
     * 
     * @param g Group to check if has any classblock.
     * @return true if the group has any classblock.
     */
    public boolean groupHasClassblocks(Group g){
		boolean exists = false;
		
		try {
			ResultSet result = statement.executeQuery("SELECT idclassblock FROM classblock WHERE  group_idgroup=" + g.getId() + ";");
			if(result.next()) {
				exists = true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
    }
    
    /**
     * Deletes a group from the DBMS
     * @param g group to delete
     */
    public void deleteGroup(Group g){
  
    	String sql="DELETE FROM " + database + ".group WHERE (idgroup='" + g.getId() +"');";
        try{
        	statement.execute(sql);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    	
    }
	
	
	/** Adds miss to the database and returns the assigned ID or 0 if there is some problem.
	 * A miss gets one ID when stored in database for the first time.*/
	public int addMiss(Miss miss) {
		int id = 0;
		
		try { 
			statement.execute("INSERT INTO miss VALUES('0', '" + miss.getType() + "', '" + dateFormat.format(miss.getDate()) + "', '" + miss.getComment() + "', '" + miss.getIDStudent() + "', '" + miss.getIDTeacher() + "', '" + miss.getIDSubject() + "', '" + miss.getIDClassblock() + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return id;
	}
	/** Update MISS TO MISS EXCUSED MISS
	 * A miss gets one ID when stored in database for the first time.*/
	public void updateExcusedtoMiss(Set<?> idmiss){
		String ids=null;
		for (Object id:idmiss){
			if (ids==null){
				ids=String.valueOf(id)+ ",";
			}else{
				ids=ids + String.valueOf(id) + ",";
			}
		}
		ids=ids.substring(0, ids.length()-1);
		try {
			statement.execute("UPDATE miss SET type=" + Miss.MISS + " WHERE idmiss IN (" + ids + ");");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	/** Update EXCUSED MISS TO MISS 
	 * A miss gets one ID when stored in database for the first time.*/
	public void updateMisstoExcused(Set<?> idmiss){
		String ids=null;
		for (Object id:idmiss){
			if (ids==null){
				ids=String.valueOf(id)+ ",";
			}else{
				ids=ids + String.valueOf(id) + ",";
			}
		}
		ids=ids.substring(0, ids.length()-1);
		try {
			statement.execute("UPDATE miss SET type=" + Miss.EXCUSED_MISS+ " WHERE idmiss IN (" + ids + ");");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	/**
	 * Returns a QueryContainer with miss type;
	 * @param type Type of the miss to look for: Miss.MISS, Miss.EXPULSION, Miss.DELAY, Miss.EXCUSSED_MISS 
	 */
	public QueryContainer getMissQueryContainer(int type,int idstudent,int idgroup,Teacher user,java.util.Date start,java.util.Date end) {
		QueryContainer querycontainer = null;
		String sqlGroupsOfTeacher=null;
		String sqlStudentsOfGroup=null;
		String sqlTerms=null;
		String sqlDates=null;
		if (idgroup==0){
			Collection groups=allGroups(user);
			for (Object g:groups){
				if (sqlGroupsOfTeacher==null){
					sqlGroupsOfTeacher=String.valueOf(((Group) g).getId())+ ",";
				}else{
					sqlGroupsOfTeacher=sqlGroupsOfTeacher + String.valueOf(((Group) g).getId())+ ",";
				}
			}
			sqlGroupsOfTeacher=sqlGroupsOfTeacher.substring(0, sqlGroupsOfTeacher.length()-1);
			sqlGroupsOfTeacher=" AND group_has_student.group_idgroup IN (" + sqlGroupsOfTeacher + ")";
		}else {
			sqlGroupsOfTeacher=" AND group_has_student.group_idgroup=" + idgroup;
		}
		
		if(idstudent==0){
			sqlStudentsOfGroup="";
		}else{
			sqlStudentsOfGroup=" AND miss.student_idstudent="+ idstudent;
		}
		if(start==null || end==null){
			sqlDates="";
		}else{
			sqlDates=" AND date BETWEEN '" +  dateFormat.format(start) + "' AND '" + dateFormat.format(end) + "'" ;
		}

		String sql=" SELECT DISTINCT idmiss,Trim(amdb.group.name) AS groupname,Trim(subject.name) AS subjectname,date ,trim(teacher.fullname) AS teachername,classblock.start,classblock.end,student.name,student.surname1,student.surname2 " +
				   " FROM miss" +
				   " INNER JOIN classblock ON classblock_idclassblock = classblock.idclassblock " +
				   " INNER JOIN group_has_student ON classblock.group_idgroup=group_has_student.group_idgroup " +
				   " INNER JOIN teacher ON teacher.idteacher=classblock.teacher_idteacher " +
				   " INNER JOIN student ON miss.student_idstudent=student.idstudent " +
				   " INNER JOIN amdb.group ON group_has_student.group_idgroup=amdb.group.idgroup " +
				   " INNER JOIN term ON classblock.term_idterm=term.idterm " +
				   " INNER JOIN subject ON classblock.subject_idsubject=subject.idsubject" +
				   " WHERE type=" + type +
				    sqlDates +
				    sqlGroupsOfTeacher +
				    sqlStudentsOfGroup;
		try {
			
			querycontainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return querycontainer;
	}
	
	/** Adds teacher to the database and returns the assigned ID or 0 if there is some problem. 
	 * A teacher gets one ID when stored in database for the first time.*/
	public int addTeacher(Teacher teacher) {
		int id = 0;
		
		try {
			int admin = teacher.getAdmin() ? 1 : 0; 
			statement.execute("INSERT INTO teacher VALUES('0', '" + teacher.getLogin() + "', '" + teacher.getPassword() + "', '" + teacher.getFullname() + "', '" + teacher.getEmail() +  "', '" + teacher.getComment() + "', '" + admin + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return id;
	}
	/** Update the specified teacher value in database. */
	public void updateTeacher(Teacher teacher,Boolean updatepassword) {
		
		try {
			int admin = teacher.getAdmin() ? 1 : 0;
			if (updatepassword){
				statement.execute("UPDATE teacher SET login='" + teacher.getLogin() + "', password='" + teacher.getPassword() + "', fullname='" + teacher.getFullname() + "', email='" + teacher.getEmail() + "', comment='" + teacher.getComment() + "', admin='" + admin + "' WHERE idteacher=" + teacher.getID() +";");
			}else{
				statement.execute("UPDATE teacher SET login='" + teacher.getLogin() + "', fullname='" + teacher.getFullname() + "', email='" + teacher.getEmail() + "', comment='" + teacher.getComment() + "', admin='" + admin + "' WHERE idteacher=" + teacher.getID() +";");
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/** Update the specified teacher value in database. */
	public void updateTeacher(Teacher teacher) {
		
		try {
			int admin = teacher.getAdmin() ? 1 : 0;
				statement.execute("UPDATE teacher SET login='" + teacher.getLogin() + "', password='" + teacher.getPassword() + "', fullname='" + teacher.getFullname() + "', email='" + teacher.getEmail() + "', comment='" + teacher.getComment() + "', admin='" + admin + "' WHERE idteacher=" + teacher.getID() +";");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/** Returns a collection of all teachers in DB*/
	public Collection<Teacher> allTeachers() {
		Vector<Teacher> vector = new Vector<Teacher>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM teacher;");
			while(result.next()) {
				vector.add(new Teacher(result.getInt("idteacher"), result.getString("login"), result.getString("password"),  result.getString("fullname"), result.getString("email"), result.getString("comment"), result.getBoolean("admin")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}
	
	/**
	 * Returns a collection of all Groups in DB
	 */
	public Collection<Group> allGroups() {
		Vector<Group> vector = new Vector<Group>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM " + database + ".group;");
			while(result.next()) {
				vector.add(new Group(result.getInt("idgroup"), result.getString("name"), result.getString("comment"),  result.getInt("teacher_idteacher")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}
    /** Returns a collection of all groups in DB from a Teacher*/
    public Collection<Group> allGroups(Teacher teacher) {
    		int idteacher=teacher.getID();
            Vector<Group> vector = new Vector<Group>();

            try {
                    ResultSet result = statement.executeQuery("SELECT * FROM amdb.group WHERE teacher_idteacher=" + idteacher);
                    while(result.next()) {
                            vector.add(new Group(result.getInt("idgroup"), result.getString("name"), result.getString("comment"), result.getInt("teacher_idteacher")));
                    }
            } catch (SQLException e) {
                    e.printStackTrace();
            }

            return vector;
    }

	
	/**
	 * Returns specified Teacher object or null
	 * 
	 * @param idteacher
	 * @return
	 */
	public Teacher getTeacher(int idteacher) {
		Teacher teacher = null;
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM teacher WHERE idteacher='" + idteacher + "';");
			while(result.next()) {
				teacher = new Teacher(result.getInt("idteacher"), result.getString("login"), result.getString("password"),  result.getString("fullname"), result.getString("email"), result.getString("comment"), result.getBoolean("admin"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return teacher;
	}
	
	/**
	 * Returns a QueryContainer with all teachers
	 *
	 */
	public QueryContainer getTeachers() {
		QueryContainer querycontainer = null;
		try {
			querycontainer = new QueryContainer("SELECT idteacher, login, password, fullname, email, comment, admin FROM teacher;", con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return querycontainer;
	}
	
	/** Deletes the specified teacher */
	public void deleteTeacher(int idteacher) {
		try {
			statement.execute("DELETE FROM teacher WHERE idteacher = " + idteacher + ";");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/** Deletes the specified miss */
	public void deleteMiss(int type, java.util.Date date, int idstudent, int idclassblock) {
		try {
			statement.execute("DELETE FROM miss WHERE type = " + type + " AND date = '" + dateFormat.format(date) + "' AND student_idstudent = " + idstudent + " AND classblock_idclassblock = " + idclassblock + ";");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns a collection of all classblocks for the specified teacher
	 * on the specified date, a classblock is only valid in a term.
	 * 
	 * The class <code>Classblock</code> has a name only used on the
	 * user interface, not stored in DBMS. The name of all returned classblocks
	 * is: start + " - " + end + " " + group.name + "/" + subject.name 
	 * 
	 * @param idtecher ID of theacher
	 * @param date 	   Date
	 * @return Collection<Classblock> with all valid classblocks for teacher on date
	 */
	public Collection<Classblock> classblocksOfTeacherOnDate(int idteacher, java.util.Date date) {
		Vector<Classblock> vector = new Vector<Classblock>();
		
		try {
			//System.out.println("classblocksOfTeacherOnDate(teacher=" + idteacher + ", date=" + date);
			//System.out.println("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend FROM classblock, term WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.teacher_idteacher=" + idteacher + " AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "') ORDER BY start;");
			ResultSet result = statement.executeQuery("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend, " + database + ".group.idgroup, " + database + ".group.name AS groupname, subject.name AS subjectname FROM classblock, term, " + database + ".group, subject WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.teacher_idteacher=" + idteacher + " AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "' AND " + database + ".group.idgroup=classblock.group_idgroup AND classblock.subject_idsubject=subject.idsubject) ORDER BY start;");
			while(result.next()) {
				Classblock block = new Classblock(result.getInt("idclassblock"), result.getInt("subject_idsubject"), result.getInt("group_idgroup"), result.getInt("day_of_week"), result.getTime("start"), result.getTime("end"));
				System.out.println("Bloque: " + block);
				block.setName(timeFormat.format(result.getTime("start")) + "-" + timeFormat.format(result.getTime("end")) + " " + result.getString("groupname") + " / " + result.getString("subjectname"));
				vector.add(block);
			}
		} catch (SQLException e) {
			Logger.log("DatabaseStorage.classblocksOfTeacherOnDate(" + idteacher + ", " + date + ")");
			e.printStackTrace();
		}
		
		return vector;
	}
	
	/** 
	 * Returns true if exists a miss for the specified type, date, classblockID
	 * and studentID
	 * 
	 * @param type Type of the miss to look for: Miss.MISS, Miss.EXPULSION, Miss.DELAY, Miss.EXCUSSED_MISS
	 * @param date	Date of miss
	 * @param classblockid	The ID of the classblock
	 * @param studentid The ID of the student
	 * @return true if the specified miss exists
	 */
	public boolean existsMiss(int type, java.util.Date date, int classblockid, int studentid) {
		Boolean exists = false;
	
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM miss WHERE type = " + type + " AND date = '" + dateFormat.format(date) + "' AND classblock_idclassblock = " + classblockid + " AND student_idstudent = " + studentid + ";");
			if(result.next()) exists = true;
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return exists;
	}
	
	/** Returns a string of lowercase hexcodes of SHA-1 hash value for the provided password */
	public String getPasswordHash(String password) {
		return hbinary.marshal( md.digest(password.getBytes()) ).toLowerCase();
	}
	
	/**
	 * Returns a QueryContainer with all groups and a count (as students)
	 *  of group's students.
	 */
	public QueryContainer getGroupsQueryContainer() {
		QueryContainer groupsQueryContainer = null;

		//String sql="SELECT amdb.group.*, COUNT(group_has_student.student_idstudent) AS students"
		//	 + " FROM amdb.group LEFT JOIN group_has_student ON amdb.group.idgroup = group_has_student.group_idgroup "
		//	 +  "GROUP BY amdb.group.idgroup;";
		//NEW SQL TABLE GROUP WITH MANAGER TEACHER
		String sql = "SELECT " + database + ".group.*, COUNT(group_has_student.student_idstudent) AS students,IFNULL(teacher.fullname,'-SIN TUTOR-') as tutor FROM " + database + ".group LEFT JOIN group_has_student ON " + database + ".group.idgroup = group_has_student.group_idgroup LEFT JOIN teacher ON teacher_idteacher = teacher.idteacher GROUP BY " + database + ".group.idgroup;";

		try {

			groupsQueryContainer = new QueryContainer(sql, con);

			//groupsQueryContainer = new QueryContainer("SELECT " + database + ".group.*, COUNT(group_has_student.student_idstudent) AS students FROM " + database + ".group LEFT JOIN group_has_student ON " + database + ".group.idgroup = group_has_student.group_idgroup GROUP BY " + database + ".group.idgroup;", con);

		} catch (SQLException e) {
			e.printStackTrace();
		}
		return groupsQueryContainer;
	}
	
	/**
	 * Returns a QueryContainer with groups of student
	 */
	public QueryContainer getGroupsOfStudentQueryContainer(Integer idStudent) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql="SELECT idgroup,name,comment from " + database + ".group,group_has_student";
			sql = sql + " WHERE idgroup=group_idgroup";
			sql = sql + " AND student_idstudent=" + idStudent + ";";
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Returns a QueryContainer with groups that not have student
	 */
	public QueryContainer getGroupsHaveNotStudentGroupQueryContainer(Integer idStudent) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql="SELECT DISTINCT idgroup,name,comment from " + database + ".group";
			sql = sql + " WHERE idgroup NOT IN (";
			sql = sql + " SELECT DISTINCT group_idgroup FROM group_has_student";
			sql = sql + " WHERE student_idstudent=" + idStudent + ");";
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Returns specified group or null if not exists.
	 * 
	 * @param idgroup
	 * @return specified group or null if not exists.
	 */
	public Group getGroup(int idgroup) {
		Group group = null;
		ResultSet result;
		
		try {
			result = statement.executeQuery("Select * from " + database + ".group where idgroup=" + idgroup + ";");
			if(result.next()) group = new Group(result.getInt("idgroup"), result.getString("name"), result.getString("comment"), result.getInt("teacher_idteacher"));

		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return group;
	}
	
	/**
	 * 
	 * Gets the name of a group.
	 * 
	 * @param idgroup ID of group to look for
	 * @return the name of group or null if not in DB
	 */
	public String getGroupName(int idgroup) {
		
		ResultSet result;
		String name = null;
		try {
			result = statement.executeQuery("Select name from " + database + ".group where idgroup=" + idgroup + ";");
			if(result.next()) name = result.getString("name");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return name;
	}
	
	/**
	 * Gets the <code>Teacher</code> assigned to a group or null.
	 * @param idgroup
	 * @return Teacher object 
	 */
	public Teacher getGroupTeacher(int idgroup) {
		Group group = getGroup(idgroup);
		if (group == null) return null;
		
		Teacher teacher = getTeacher(group.getIdteacher());
		if (teacher == null) return null;
		
		return teacher;
	}
	
	/**
	 * Returns a QueryContainer with all students
	 */
	public QueryContainer getStudentsQueryContainer() {
		QueryContainer studentsQueryContainer = null;
		try {
			studentsQueryContainer = new QueryContainer("SELECT * FROM student;", con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Returns a QueryContainer with all terms
	 */
	public QueryContainer getTerms() {
		QueryContainer termsQueryContainer = null;
		try {
			termsQueryContainer = new QueryContainer("SELECT * FROM term;", con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return termsQueryContainer;
	}
	
	/**
	 *  Returns a collection of all terms in DB
	 * */
	public Collection<Term> allTerms() {
		Vector<Term> vector = new Vector<Term>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM term;");
			while(result.next()) {
				vector.add(new Term(result.getInt("idterm"), result.getString("name"), result.getString("comment"),result.getDate("start"),result.getDate("end")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}
	/**
	 * Add  term
	 */
	public Integer addTerm(Term term) {
		int id = 0;
		
		try {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			statement.execute("INSERT INTO term VALUES(" +
					"'0', '" + term.getName() + "', '" + term.getComment() 
					+ "', '" + sdf.format(term.getStart()) + "', '" + sdf.format(term.getEnd()) + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return id;
	}
	
	/**
	 * Update term information
	 */
	public Integer updateTerm(Term term) {
		
		Integer id=term.getIdTerm();
		String name=term.getName();
		String comment=term.getComment();
		java.util.Date start=term.getStart();
		java.util.Date end=term.getEnd();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String sql="UPDATE term SET name='" + name +"' , comment='"+ comment + "'," + 
        	"start='" +  sdf.format(start) +"',end='"+ sdf.format(end) +
        	"' WHERE idterm=" + id + ";";
        int res=0;
        try{
                res = statement.executeUpdate(sql);
        } catch (SQLException e) {
                e.printStackTrace();
        }

        return res;
	}
	
	/** Deletes the specified term */
	public void deleteTerm(int idterm) {
		try {
			statement.execute("DELETE FROM term WHERE idterm = " + idterm + ";");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/** Returns a collection of subjects*/
	public Collection<Subject> allSubjects() {
		Vector<Subject> vector = new Vector<Subject>();
		
		try {
			ResultSet result = statement.executeQuery("SELECT * FROM subject;");
			while(result.next()) {
				vector.add(new Subject(result.getInt("idsubject"), result.getString("name"), result.getString("comment")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
	}

	public QueryContainer getSubjects() {
		QueryContainer querycontainer = null;
		try {
			querycontainer = new QueryContainer("SELECT * FROM subject;", con);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return querycontainer;
	}
	
	/**
	 * Add  Subject
	 */
	public int addSubject(Subject subject) {
		int id = 0;
		try { 

			statement.execute("INSERT INTO subject VALUES('0', '" +  subject.getName() + "', '" + subject.getComment() +  "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");
			System.out.println(id);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return id;
	}
	
	/**
	 * Refresh table data from a QueryContainer. The previous content of table
	 * is removed and new data is loaded from the provided QueryContainer.
	 * 
	 * The value of the first property of QueryContainer will be used as
	 * Item ID in table.
	 * 
	 * @param qc 			A QueryContainer with data to load
	 * @param table			Table to refresh
	 * @param properties	Array of properties IDs to use
	 */
	public void refreshTable(QueryContainer qc, Table table) {
		table.removeAllItems();
		
		int qcsize = qc.size();
		if (qcsize == 0) {
			Logger.log("DatabaseStorage.refreshTable() empty query container");
			return;
		}
		
		List<Object> propertyIds = new ArrayList<Object>(qc.getContainerPropertyIds());
		int propertiesnumber = propertyIds.size();
		if (propertiesnumber <= 1) {
			Logger.log("DatabaseStorage.refreshTable() query container needs more than one property");
			return;
		}
		
		for(int i=1; i<=qcsize; i++) {
			Item dataItem = qc.getItem(i);
			Item tableItem = table.addItem(dataItem.getItemProperty(propertyIds.get(0)).getValue());
			if (tableItem == null) {
				Logger.log("DatabaseStorage.refreshTable() couldn't add new table's item");
			}
			for(int j=1; j<propertiesnumber; j++) {
				Object pid = propertyIds.get(j);
				//System.out.println("DatabaseStorage.refreshTable() pid: " + pid);
				tableItem.getItemProperty(pid).setValue(dataItem.getItemProperty(pid).getValue());
			}
		}

	}
	
	/**
	 * Adds the specified student to database.
	 * 
	 * @param student Student object
	 */
	public int addStudent(Student student) {
		int id=0;
		try {
		statement.executeUpdate("INSERT INTO student(surname1,surname2,name,email,smsphone) " 
					+ " VALUES('" 
					+ student.getSurname1() + "','" 
					+ student.getSurname2() + "','"
					+ student.getName() + "','"
					+ student.getEmail() + "','"
					+ student.getSmsphone() + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");		
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return id;
	}
	
	/**
	 * Adds the specified studentid to the group with groupid.
	 * 
	 * @param studentid	ID of student
	 * @param groupid	ID of group 
	 */
	public void addStudentToGroup(int studentid, int groupid) {
		try {
			statement.executeUpdate("INSERT INTO group_has_student VALUES('" + groupid +"','" + studentid + "');");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Update values Subject
	 */
	public int updateSubject(Subject subject) {
		Integer id=subject.getID();
		String name=subject.getName();
		String comment=subject.getComment();
        String sql="UPDATE subject SET name='" + name +"' , comment='"+ comment + 
        	"' WHERE idsubject=" + id + ";";
        int res=0;
        try{
                res = statement.executeUpdate(sql);
        } catch (SQLException e) {
                e.printStackTrace();
        }
        return res;
	}
	
    /** Deletes the specified subject */
    public void deleteSubject(Integer idsubject ) {
            try {
                    statement.execute("DELETE FROM subject WHERE idsubject = " + idsubject +  ";");
            } catch (SQLException e) {
                    e.printStackTrace();
            }
    }

    /** Update Student Information*/
    public int updateStudent(Student student){
        	int res=0;
    		Integer id=student.getID();
    		String name=student.getName();
    		String surname1=student.getSurname1();
    		String surname2=student.getSurname2();
    		String email=student.getEmail();
    		String smsphone=student.getSmsphone();
            String sql="UPDATE student SET name='" + name +"' ," 
            		+ " surname1='"+ surname1 + "',"
            		+ " surname2='"+ surname2 + "',"
            		+ " email='"+ email + "',"
            		+ " smsphone='"+ smsphone + "'"
            		+ " WHERE idstudent=" + id + ";";
            try{
                    res = statement.executeUpdate(sql);
            } catch (SQLException e) {
                    e.printStackTrace();
            }

            return res;
    }
    
    /**
	 * Delete Subject
	 */
	public void deleteSubject(int idSubject){
		try {
			statement.execute("DELETE FROM subject WHERE idsubject="+ idSubject); 
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
    
	/**
	 * Delete Student
	 */
	public void deleteStudent(Student student){
		try {
			statement.execute("DELETE FROM student WHERE idstudent="+ student.getID()); 
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Delete Student to Group
	 */
	public void deleteStudentFromGroup(Integer idgroup,Integer idstudent){
		try {
			statement.execute("DELETE FROM group_has_student WHERE group_idgroup=" + idgroup + " AND student_idstudent=" + idstudent ); 
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Returns a QueryContainer with students of group
	 */
	public QueryContainer getStudentsOfGroupQueryContainer(Integer idgroup) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql=" SELECT idstudent,surname1,surname2,name,email,smsphone from student s,group_has_student";
			sql = sql + " WHERE idstudent=student_idstudent";
			sql = sql + " AND group_idgroup=" + idgroup + ";";
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Returns a QueryContainer with students fullname  of group
	 * If idgroup=0 return all students
	 */
	public Collection getStudentsCollection(Integer idgroup) {
		Vector<Student> vector = new Vector<Student>();
		String sql="";
		if (idgroup==0){
			sql=" SELECT DISTINCT idstudent,surname1,surname2,name,email,smsphone";
			sql = sql + " FROM student,group_has_student";
			sql = sql + " WHERE idstudent=student_idstudent";
			sql = sql + " AND group_idgroup<>" + idgroup;
			sql = sql + " ORDER BY surname1;";
			
		}else{
			sql=" SELECT DISTINCT idstudent,surname1,surname2,name,email,smsphone";
			sql = sql + " FROM student,group_has_student";
			sql = sql + " WHERE idstudent=student_idstudent";
			sql = sql + " AND group_idgroup=" + idgroup;
			sql = sql + " ORDER BY surname1;";
		
		}
		

		try {
			ResultSet result = statement.executeQuery(sql);
			while(result.next()) {
				vector.add(new Student(result.getInt("idstudent"), result.getString("surname1"), result.getString("surname2"),result.getString("name"),result.getString("email"),result.getString("smsphone")));
			}
		} catch (SQLException e) {
				e.printStackTrace();
			}
			
			return vector;
		
	}
	/**
	 * Returns a QueryContainer with students of group
	 */
	public QueryContainer getStudentsNotInGroupQueryContainer(Integer idgroup) {
		QueryContainer studentsQueryContainer = null;
		try {
			String sql="SELECT DISTINCT idstudent,surname1,surname2,name,email,smsphone from student";
			sql = sql + " WHERE idstudent NOT IN (";
			sql = sql + " SELECT DISTINCT student_idstudent FROM group_has_student";
			sql = sql + " WHERE group_idgroup=" + idgroup + ");";
			
			studentsQueryContainer = new QueryContainer(sql, con);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return studentsQueryContainer;
	}
	
	/**
	 * Removes the specified studentid from the group with groupid.
	 * 
	 * @param studentid	ID of student
	 * @param groupid	ID of group 
	 */
	public void removeStudentFromGroup(int studentid, int groupid) {
		try {
			statement.executeUpdate("DELETE FROM group_has_student WHERE (group_has_student.group_idgroup='" + groupid + "' AND group_has_student.student_idstudent='" + studentid + "');");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	//
	// Código para generar los informes y las notificaciones SMS/Mail
	// 
	
	/**
	 * Returns a <code>ResultSet</code> with count of misses in a given group.
	 * 
	 * The result set has columns:
	 * | idstudent | surname1 | surname2 | name     | smsphone | email               | misscount | excusedcount | delaycount | expulsioncount |
	 * <pre>
	 * +-----------+----------+----------+----------+----------+---------------------+-----------+--------------+------------+----------------+
     * | idstudent | surname1 | surname2 | name     | smsphone | email               | misscount | excusedcount | delaycount | expulsioncount |
     * +-----------+----------+----------+----------+----------+---------------------+-----------+--------------+------------+----------------+
     * |         1 | student1 | student1 | student1 | 12345    | student1@server.com |         3 |            0 |          0 |              0 |
     * |         2 | student2 | student2 | student2 | 12345    | student2@server.com |         0 |            1 |          0 |              0 |
     * |         3 | student3 | student3 | student3 | 12345    | student3@server.com |         2 |            1 |          0 |              0 |
     * |         5 | student5 | student5 | student5 | 12345    | student5@server.com |         1 |            1 |          0 |              0 |
     * |         6 | student6 | student6 | student6 | 12345    | student1@server.com |         1 |            0 |          0 |              0 |
     * |         7 | student7 | student7 | student7 | 12345    | student2@server.com |         0 |            0 |          3 |              0 |
     * |         8 | student8 | student8 | student8 | 12345    | student3@server.com |         0 |            0 |          2 |              0 |
     * |         9 | student9 | student9 | student9 | 12345    | student4@server.com |         0 |            0 |          1 |              2 |
     * +-----------+----------+----------+----------+----------+---------------------+-----------+--------------+------------+----------------+
	 * </pre>
	 * 
	 * @param idgroup ID of group to report
	 * @param dateA   Begin date for miss count
	 * @param dateB   End date for miss count
	 * 
	 * @return ResultSet with result data or null
	 */
	public ResultSet studentReport1(int idgroup, Date dateA, Date dateB) {
		
		ResultSet result = null;
		try {
			result = statement.executeQuery("SELECT student.idstudent, student.surname1, student.surname2, student.name, student.smsphone, student.email, sum(if (miss.type=1,1,0)) 'misscount', sum(if (miss.type = 2,1,0)) 'excusedcount', sum(if (miss.type =3,1,0)) 'delaycount',sum(if (miss.type = 4,1,0)) 'expulsioncount' FROM student INNER JOIN miss ON miss.student_idstudent=student.idstudent INNER JOIN group_has_student ON group_has_student.student_idstudent=student.idstudent WHERE group_idgroup=" + idgroup + " AND miss.date BETWEEN '" + dateFormat.format(dateA) + "' AND '" + dateFormat.format(dateB) + "' GROUP BY student.idstudent ORDER BY student.surname1, student.surname2, student.name;");
		} catch (SQLException e) {
			e.printStackTrace();
		}
	       
	    return result;
	}
	
	/**
	 * 
	 * ESTE MÉTODO DEBE CAMBIAR PARA ACEPTAR DOS Date COMO PARÁMETRO
	 * EN LUGAR DE DOS String PARA LAS FECHAS !!!!
	 * 
	 * @param idgroup
	 * @param dateA
	 * @param dateB
	 * @return
	 */
	public ResultSet studentReport2(int idgroup, Date dateA, Date dateB) {
		
		ResultSet result = null;
		try {
			result = statement.executeQuery("SELECT student.idstudent, student.surname1, student.surname2, student.name, subject.name 'subject',sum(if (miss.type=1,1,0)) 'misscount', sum(if (miss.type = 2,1,0)) 'excusedcount',         sum(if (miss.type =3,1,0)) 'delaycount',sum(if (miss.type = 4,1,0)) 'expulsioncount'   FROM   student INNER JOIN miss ON miss.student_idstudent=student.idstudent INNER JOIN group_has_student ON group_has_student.student_idstudent=student.idstudent INNER JOIN subject on miss.subject_idsubject=subject.idsubject WHERE group_idgroup= "+idgroup+" AND miss.date BETWEEN '"+dateA+"'AND '"+dateB+"' GROUP BY student.idstudent ORDER BY student.surname1, student.surname2, student.name, subject_idsubject;");
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
				
		return result;
	}

	/**
	 * Returns a QueryContainer with Classblocks
	 * // It's a name composed by time + group.name + " / " + subject.name
	 * */
    public QueryContainer getClassblocks() {
        QueryContainer querycontainer = null;
        app = new AttendancemanagerApplication();
        String sql;
                sql="SELECT classblock.idclassblock," 
                + "(CASE classblock.day_of_week "
                + " WHEN 1 THEN "  + "'"+app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_SUNDAY")+"'"
                + " WHEN 2 THEN "  + "'"+app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_MONDAY")+"'"
                + " WHEN 3 THEN "  + "'"+app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_TUESDAY")+"'"
                + " WHEN 4 THEN "  + "'"+app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_WEDNESDAY")+"'"
                + " WHEN 5 THEN "  + "'"+app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_THURSDAY")+"'"
                + " WHEN 6 THEN "  + "'"+app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_FRIDAY")+"'"
                + " WHEN 7 THEN "  + "'"+app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_SATURDAY")+"'"
                + " END) AS dayofweekstring"
                + " ,classblock.day_of_week,classblock.teacher_idteacher,classblock.subject_idsubject,classblock.group_idgroup,classblock.term_idterm,Trim(teacher.fullname) AS teachername,Trim(term.name) as termname,Trim(amdb.group.name) as groupname,Trim(subject.name) AS subjectname,classblock.start, classblock.end" 
                + " FROM classblock "
                + " INNER JOIN teacher ON classblock.teacher_idteacher=teacher.idteacher"
                + " INNER JOIN term ON classblock.term_idterm=term.idterm"
                + " INNER JOIN amdb.group ON classblock.group_idgroup=amdb.group.idgroup"
                + " INNER JOIN subject ON classblock.subject_idsubject=subject.idsubject"
                + " ORDER BY classblock.idclassblock";

        try {
                querycontainer = new QueryContainer(sql, con);;
        } catch (SQLException e) {
                e.printStackTrace();
        }
        return querycontainer;
    }

    public Integer addClassblock(Classblock classblock){
		Integer id=0;
		try {
		SimpleDateFormat sdf= new SimpleDateFormat("HH:mm");
		statement.executeUpdate("INSERT INTO classblock  " 
					+ " VALUES(" 
					+ 0 + ",'"
					+ classblock.getIdteacher() + "','" 
					+ classblock.getIDSubject()+ "','"
					+ classblock.getIdgroup()+ "','"
					+ classblock.getIdterm()+ "','"
					+ classblock.getDay_of_week() + "','"
					+ sdf.format(classblock.getStart()) + "','"
					+ sdf.format(classblock.getEnd()) + "');");
			ResultSet result = statement.executeQuery("SELECT LAST_INSERT_ID() AS ID;");
			result.next();
			id = result.getInt("ID");		
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return id;
	}
	/**
	 * Update term information
	 */
	public Integer updateClassblock(Classblock classblock) {
		Integer idClassblock=classblock.getID();
		Integer idTeacher =classblock.getIdteacher();
		Integer idTerm =classblock.getIdterm();
		Integer idGroup=classblock.getIdgroup();
		Integer idSubject=classblock.getIDSubject();
		Integer idDayofweek=classblock.getDay_of_week();
		java.util.Date hourStart=classblock.getStart();
		java.util.Date hourEnd=classblock.getEnd();
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        String sql="UPDATE classblock SET "
        	       + " teacher_idteacher=" + idTeacher + "," 
        	       + " subject_idsubject="+ idSubject + ","
        	       + " group_idgroup="+ idGroup + ","
        	       + " term_idterm="+ idTerm + ","
        	       + " day_of_week="+ idDayofweek + ","
        	       + " start='" +  sdf.format(hourStart)+ "',"
        	       + " end='"+ sdf.format(hourEnd) + "'"
        	       + " WHERE idclassblock=" + idClassblock + ";";
        int res=0;
        
        try{
                res = statement.executeUpdate(sql);
        } catch (SQLException e) {
                e.printStackTrace();
        }

        return res;
	}
	/**
	 * Delete Classblock
	 */
	public void deleteClassblock(Integer idclassblock){
		try {
			statement.execute("DELETE FROM classblock WHERE idclassblock="+ idclassblock); 
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public Collection<Classblock> getClassblocksOnDate(int idgroup, Date date){
		Vector<Classblock> vector = new Vector<Classblock>();
		
		try {
			//System.out.println("classblocksOfTeacherOnDate(teacher=" + idteacher + ", date=" + date);
			//System.out.println("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend FROM classblock, term WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.teacher_idteacher=" + idteacher + " AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "') ORDER BY start;");
			ResultSet result = statement.executeQuery("SELECT classblock.*, term.idterm, term.start AS tstart, term.end AS tend, " + database + ".group.idgroup, " + database + ".group.name AS groupname, subject.name AS subjectname FROM classblock, term, " + database + ".group, subject WHERE (classblock.day_of_week=DAYOFWEEK('" + dateFormat.format(date) + "') AND classblock.group_idgroup=" + idgroup + " AND classblock.term_idterm=term.idterm AND term.start<='" + dateFormat.format(date) + "' AND term.end>='" + dateFormat.format(date) + "' AND " + database + ".group.idgroup=classblock.group_idgroup AND classblock.subject_idsubject=subject.idsubject) ORDER BY start;");
			while(result.next()) {
				Classblock block = new Classblock(result.getInt("idclassblock"), result.getInt("subject_idsubject"), result.getInt("group_idgroup"), result.getInt("day_of_week"), result.getTime("start"), result.getTime("end"));
				System.out.println("Bloque: " + block);
				block.setName(timeFormat.format(result.getTime("start")) + "-" + timeFormat.format(result.getTime("end")) + " " + result.getString("groupname") + " / " + result.getString("subjectname"));
				vector.add(block);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return vector;
		
		
	}
}
