/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

public class Group implements Entity, HasGetName {

	private int id;
	private String name = null;
	private String comment;
	private int idteacher;
	
	/**
	 * Constructor de los nuevos Group. Al añadir un Group desde la aplicación,
	 * se usará este constructor.
	 * @param id
	 * @param name
	 * @param comment
	 * @param idteacher
	 */
	public Group(int id, String name, String comment, int idteacher) {

		this.id = id;
		this.idteacher=idteacher;
		
		if (name.equals("")) {
			Logger.log("Group.Group(" + id + " ," + name + " ," + comment + " ," + idteacher + ")");
			Logger.log("No se puede crear un grupo con el nombre '' !!!!");
		} else {
			setName(name);
		}
		
		if(comment.equals("")) setComment(null); else setComment(comment);
	}
	
	/**
	 * Constructor de los Group importados. Al importar un Group desde un csv,
	 * se usará este constructor.
	 * @param name
	 * @param comment
	 */
	public Group(String name, String comment) {
		this(0, name, comment, 0);
	}

	/**
	 * Método estático que devuelve un array con las cabeceras del csv
	 * @return devuelve un array con las cadenas de las cabeceras que deberían tener los csv de Group
	 */
	public static String[] getBasicProperties() {
		String columns[] =  {"name", "comment"};
		return columns;
	}
	
	public int getId() {
		return id;
	}
	
	public void setId(int id) {
		this.id = id;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = Constants.shortString(name);
	}
	
	public String getComment() {
		return comment;
	}
	
	public void setComment(String comment) {
		this.comment = Constants.longString(comment);
	}

	/**
	 * Se añade él mismo a la base de datos
	 * @return el id del Group insertado
	 */
	public int add() {
		return storage.addGroup(this);
	}
	
	/**
	 * Se elimina él mismo de la base de datos
	 */
	public void delete() {
		 storage.deleteGroup(this);
	}

	/**
	 * Se modifica él mismo en la base de datos
	 * @return el código de retorno
	 */
	public int update() {
		return storage.updateGroup(this);
	}
	
	@Override
	public String toString() {
		return "Group [id=" + id + ", name=" + name + ", comment=" + comment
				+ ", idteacher=" + idteacher + "]";
	}

	public int getIdteacher() {
		return idteacher;
	}

	public void setIdteacher(int idteacher) {
		this.idteacher = idteacher;
	}

}
