/*
AttendanceManager
Copyright (C) 2010  Victor Carceler

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

/*
 *  Author: Sergio Santamaría <santamaria.sergio@gmail.com>
* 
* Mailer class improve sending mails through AttendanceManager app*/

package org.iespuigcastellar.attendancemanager.core;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.activation.DataHandler;
import javax.mail.Address;
import javax.mail.BodyPart;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.SendFailedException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.util.ByteArrayDataSource;

/**
 *  The class <code>Mailer</code> improve sending mails with attached documents.
 *  
 *  @author Sergio Santamaría <santamaria.sergio@gmail.com>
 *  */

public class Mailer {
	
	private ResourceBundle rb;
	public String host;
	public String from;
	public String[] to;
	public String subject;
	public String bodyText;
	public byte[] byteArray;
	public String mimeType;
	public String filename;
	public Collection<MimeBodyPart> attachments = new Vector<MimeBodyPart>();
	
	public Mailer(){
		rb = ResourceBundle.getBundle("AttendanceManager");
		this.host = rb.getString("mailer_host");
		this.from = rb.getString("mailer_from");
	}
	
	public Mailer(String host, String from, String[] to, String subject, String bodyText){
		this.host = host;
		this.from = from;
		this.to = to;
		this.subject= subject;
		this.bodyText = bodyText;
	}
	
	
	/**
	 *  The method <code>getHost()</code> get the MTA host server.
	 *  
	 *  @return host <code>String</code> the server's name that has Mail Transfer Agent service on
	 *  */
	public String getHost(){
		return host;
	}
	/**
	 *  The method <code>getBodyText()</code> get the text message part of the mail.
	 *  
	 *  @return bodyText <code>String</code> the text Bodypart of the mail
	 *  */
	public String getBodyText(){
		return bodyText;       
	}
	/**
	 *  The method <code>getFrom()</code> get the from adress of the mail.
	 *  
	 *  @return from <code>String</code> Internet Adress of the mail's sender.
	 *  */
	public String getFrom(){
		return from;
	}
	/**
	 *  The method <code>getSubject()</code> get the mail's subject.
	 *  
	 *  @return subject <code>String</code> the mail's subject.
	 *  */
	public String getSubject(){
		return subject;
	}
	/**
	 *  The method <code>getTo()</code> get the String[] of to adresses.
	 *  
	 *  @return to <code>String[]</code> Array of Internet Adresses of mail's receivers
	 *  */
	public String[] getTo(){
		return to;
	}
	/**
	 *  The method <code>getToElements()</code> get all adresses of the String[] to adresses. 
	 *  
	 *  @return toElements <code>String</code> all the Internet Adresses of the mail's receivers
	 *  */
	public String getToElements(){
		String toElements="";
		for (int i=0;i<to.length;i++){
			toElements= toElements+to[i]+", ";
		}
		toElements = toElements.substring(0, toElements.length()-2);
		return toElements;
	}
	/**
	 *  The method <code>getMailAttachment()</code> get the byte[] of the file attached in mail. 
	 *  
	 *  @return byteArray <code>byte[]</code> the file parsed in mail's attachment
	 *  */
	public byte[] getMailAttachment(){
		return byteArray;
	}
	/**
	 *  The method <code>getMimeType()</code> get the Mimetype of the file attached in mail. 
	 *  
	 *  @return mimeType <code>String</code> the Mimetype of mail's file attached
	 *  */
	public String getMimeType(){
		return mimeType;
	}
	/**
	 *  The method <code>getFilename()</code> get the filename of the file attached in mail. 
	 *  
	 *  @return filename <code>String</code> the mail's filename attachment
	 *  */
	public String getFilename(){
		return filename;
	}
	/**
	 *  The method <code>setHost()</code> set the MTA host server. 
	 *  
	 *  @param host <code>String</code> the server's name that has Mail Transfer Agent service on
	 *  */
	public void setHost(String host){ 
		this.host = host;
	}
	/**
	 *  The method <code>setBodyText()</code> set the text message part of the mail.
	 *  
	 *  @param bodyText <code>String</code> the text Bodypart of the mail
	 *  */
	public void setBodyText(String bodyText)
	{
		this.bodyText = bodyText;
	}
	/**
	 *  The method <code>setFrom()</code> set the from adress of the mail.
	 *  
	 *  @param from <code>String</code> Internet Adress of the mail's sender.
	 *  */
	public void setFrom(String from)
	{
		this.from = from;
	}
	/**
	 *  The method <code>setSubject()</code> set the mail's subject.
	 *  
	 *  @param subject <code>String</code> the mail's subject.
	 *  */
	public void setSubject(String subject){
		this.subject = subject;
	}
	/**
	 *  The method <code>setTo()</code> set all adresses of the String[] to adresses. 
	 *  
	 *  @param to <code>String[]</code> Array of Internet Adresses of mail's receivers
	 *  */
	public void setTo(String[] to){
		this.to = to;
	}
	/**
	 *  The method <code>addMailAttachment()</code> creates a MimeBodyPart with an attachment and add it to a Collection of attachments(MimeBodyPart).
	 *    
	 *  @param filename <code>String</code> the mail's filename attachment
	 *  @param byteArray <code>byte[]</code> the file parsed in mail's attachment
	 *  @param mimeType <code>String</code> the Mimetype of mail's file attached 
	 *  
	 *  */
	public void addMailAttachment(byte[] byteArray, String mimeType, String filename){
		this.byteArray = byteArray;
		this.mimeType = mimeType;
		this.filename = filename;		
		MimeBodyPart attachBodyPart = new MimeBodyPart();
		try{				
			attachBodyPart.setDataHandler(new DataHandler(
					new ByteArrayDataSource(byteArray,mimeType)));
			// Set the name of the file attached
		    attachBodyPart.setFileName(filename);
		    
			//setHeader defines the properties of the part
		    attachBodyPart.setHeader("MIME-Version", "1.0");
		    attachBodyPart.setHeader("Content-Transfer-Encoding", "base64");
		    attachments.add(attachBodyPart);			    
		    
		}catch(Exception ex){
			ex.printStackTrace();
		}
	}
	
	/**
	  * The method <code>isValidEmailAddress</code> validate the form of an email address.
	  *
	  * <P>Return true only if <code>address</code> can successfully construct an javax.mail.internet.InternetAddress 
	  * 
	  * @param address <code>String</code> Email Adress to checkout
	  * @return boolean
	  */
	  public boolean isValidEmailAddress(String address){
	    if (address == null) return false;
	    boolean result = true;
	    try {
	      InternetAddress emailAddr = new InternetAddress(address);	
	      emailAddr.validate();
	    }
	    catch (AddressException ex){
	      result = false;
	    }
	    return result;
	  }
	
	/**
	 *  The method <code>toString()</code> returns a String formed with the properties of the mail
	 *  
	 *  @return str <code>String</code> the complete properties of the mail parsed to a String 
	 *  */
	@Override
	public String toString(){
		
		String str = "Mail Properties: \n * host: "+this.getHost()+ "\n * from: "+this.from+ "\n * to: "+this.getToElements()+"\n * subject: "+this.getSubject()
		+"\n * body: "+this.getBodyText();
		if (!this.attachments.isEmpty()) str=str+"\n * mimeType: "+this.getMimeType()+"\n * filename: "+this.getFilename();
		return str;
		
	}
	/**
	 *  The method <code>sendMail()</code> sends a mail with the seted properties
	 *  
	 *  You must add at minum a host, from adress, to adress, subject and bodytext if you want to use this method.
	 *  Look at "see" methods
	 *  
	 *  @see <code>setHost()</code>
	 *  @see <code>setFrom()</code>
	 *  @see <code>setTo()</code>
	 *  @see <code>setSubject()</code>
	 *  @see <code>setBodyText()</code>
	 * 
	 *  */
	public void sendMail() {
		// Checkout if the minimum content of message is set
		if (this.getHost()==null ||
			this.getFrom()==null ||
			this.getTo()==null ||
			this.getSubject()==null ||
			this.getBodyText()==null) {
			Logger.log("Mailer() ERROR:\n Mail can't be sended without a host, from adress, to adress, subject or bodytext");
			
		}
		else{
			try{
				// Set file encoding of JVM to UTF-8
				System.setProperty("file.encoding", "UTF-8");
				
				// Set mail.mime.charset
				System.setProperty("mail.mime.charset","UTF-8");
				
				// Test charset encoding
				//String encoding = (new InputStreamReader(System.in)).getEncoding();
				//System.out.println("System Encoding: "+encoding);
				//System.out.println("Default MIME Charset:"+System.getProperty("mail.mime.charset"));
				//System.out.println("Default Java Charset:"+MimeUtility.getDefaultJavaCharset());

				// Get system properties
				Properties props = System.getProperties();
				props.put("mail.smtp.host", this.getHost());
				
				/**
				 * The <code>Session</code> defines a basic mail session. It is through this session that everything else works.
				 * The <code>Session</code> object takes advantage of a <code>java.util.Properties</code> object to get information like mail server, 
				 * username, password, and other information that can be shared across your entire application.
				 * 
				 *  You can get a single default session that can be shared with the <code>getDefaultInstance()</code> method:
				 * <code>Session session = Session.getDefaultInstance(props, null);</code>
				 * or create a unique session
				 * <code>Session session = Session.getInstance(props, null);</code>
				 * */
				
				// Get session
				Session session = Session.getDefaultInstance(props, null);
				// Create a message
				MimeMessage message = new MimeMessage(session);
			    
				message.setFrom(new InternetAddress(this.getFrom()));
			    setAddress(message, this.getTo(), Message.RecipientType.TO);
			    message.setSubject(this.getSubject());
			    
			    Multipart multipart = new MimeMultipart();
			    
			    // Text message Body Part of message
			    MimeBodyPart messageBodyPart = new MimeBodyPart();
			    messageBodyPart.setText(this.getBodyText(),"UTF-8");
				//setHeader defines the properties of the part
				messageBodyPart.setHeader("MIME-Version", "1.0");
				messageBodyPart.setHeader("Content-Type", "text/plain; charset='UTF-8'");
				multipart.addBodyPart(messageBodyPart);         
			    
			    //Add Attachment parts
				// If the mail has attachments add to multipart  attachments.add(attachBodyPart);
				
				if (this.attachments != null){
					Iterator itr = this.attachments.iterator();
					while(itr.hasNext()){
						multipart.addBodyPart((MimeBodyPart) itr.next());					
					}				
				}
			    		    		    		    
			    //Set the Sent Date
			    message.setSentDate(new Date());
			    
			    //Add the Multipart to the message
				message.setContent(multipart);
		
				//Set the Date: header
				message.saveChanges(); 
				
				/**
				 * <code>Transport</code> speaks the protocol-specific language for sending the message (usually SMTP). 
				 * It's an abstract class and works something like Session. You can use the default version of 
				 * the class by just calling the static send() method: 
				 * <code>Transport.send(msg);</code>
				 * 
				 * Or, you can get a specific instance from the session for your protocol, pass along the username
				 * and password (blank if unnecessary), send the message, and close the connection:
				 * 
				 * <code>msg.saveChanges(); // implicit with send()
				 * Transport transport = session.getTransport("smtp");
				 * transport.connect(host, username, password);
				 * transport.sendMessage(message, message.getAllRecipients());
				 * transport.close();</code> */
				
				// Send message
				Transport.send(message);
				Logger.log("Mailer() Mail SENDED\n"+ this.toString());
				System.out.println("Mailer() Mail SENDED\n"+this.toString());
			    			
			}
			catch (Exception e) {
				Logger.log("Mailer() Mail NOT SENDED\n"+ this.toString());
				System.out.println("Mailer() Mail NOT SENDED\n"+this.toString());
			    System.out.println("\n--Exception handling in Mailer.java");
			    
			    //Handling Exceptions
			    
			    e.printStackTrace();
			    System.out.println();
			    Exception ex = e;
			    do {
			    	//Exception sending mail: this takes the invalid addresses if any
				if (ex instanceof SendFailedException) {
				    SendFailedException sfex = (SendFailedException)ex;
				    Address[] invalid = sfex.getInvalidAddresses();
				    if (invalid != null) {
					System.out.println("    ** Invalid Addresses");
						if (invalid != null) {
						    for (int i = 0; i < invalid.length; i++)
							System.out.println("         " + invalid[i]);
						}
				    }
				    Address[] validUnsent = sfex.getValidUnsentAddresses();
				    if (validUnsent != null) {
					System.out.println("    ** ValidUnsent Addresses");
						if (validUnsent != null) {
						    for (int i = 0; i < validUnsent.length; i++)
							System.out.println("         "+validUnsent[i]);
						}
				    }
				    Address[] validSent = sfex.getValidSentAddresses();
				    if (validSent != null) {
					System.out.println("    ** ValidSent Addresses");
						if (validSent != null) {
						    for (int i = 0; i < validSent.length; i++)
							System.out.println("         "+validSent[i]);
						}
				    }
				}
				System.out.println();
				// If there are more exceptions than sending mail are listed here
				if (ex instanceof MessagingException)
				    ex = ((MessagingException)ex).getNextException();
				else
				// Ignore the rest of exceptions
				    ex = null;
			    } while (ex != null);
			}
			
		}
		
		
	}
	/**This method <code>setAddress()</code> is called by the sendMail method to set to address
	 * 
	 * @param message <code>Message</code> the message to set the address
	 * @param sAddressArray <code>String[]</code>  InternetAddress of the receivers of the mail (toAdresses)
	 * @param type <code>Message.RecipientType</code> the type of the destination address (TO,CC,BCC)
	 */	
	private void setAddress(Message message, String[] sAddressArray, Message.RecipientType type) throws Exception {
       if (sAddressArray != null){
    	   InternetAddress[] addressTo = new InternetAddress[sAddressArray.length];
    	   for(int i = 0; i < sAddressArray.length; i++){
    		   addressTo[i] = new InternetAddress(sAddressArray[i]);
    	   }
    	   message.setRecipients(type, addressTo);
       }
	}	
}
	
