/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

public class Student implements Entity, HasGetName {
	private int id;
	private String surname1;
	private String surname2;
	private String name = null;
	private String email;
	private String smsphone;
	
	/**
	 * Constructor de los nuevos Student. Al añadir un Student desde la aplicación,
	 * se usará este constructor.
	 * @param id
	 * @param surname1
	 * @param surname2
	 * @param name
	 * @param email
	 * @param smsphone
	 */
	public Student(int id, String surname1, String surname2, String name, String email, String smsphone) {
		this.id = id;
		if (surname1.equals("")) setSurname1(null); else setSurname1(surname1);
		if (surname2.equals("")) setSurname2(null); else setSurname2(surname2);
		
		if (name.equals("")) {
			Logger.log("Student.Student(" + id + " ," + surname1 + " ," + surname1 + " ," + name + " ," + email + " ," + smsphone+ ")");
			Logger.log("No se puede crear un alumno con el nombre '' !!!!");
		} else {
			setName(name);
		}
		
		if (email.equals("")) setEmail(null); else setEmail(email);
		if (smsphone.equals("")) setSmsphone(null); else setSmsphone(smsphone);
	}
	
	/**
	 * Constructor de los Student importados. Al importar un Student desde un csv,
	 * se usará este constructor.
	 * @param surname1
	 * @param surname2
	 * @param name
	 * @param email
	 * @param smsphone
	 */
	public Student(String surname1, String surname2, String name, String email, String smsphone){
		this(0, surname1, surname2, name, email, smsphone);
	}
	
	/**
	 * Método estático que devuelve un array con las cabeceras del csv
	 * @return devuelve un array con las cadenas de las cabeceras que deberían tener los csv de Student
	 */
	public static String[] getBasicProperties() {
		String columns[] =  {"surname1", "surname2", "name", "email", "smsphone"};
		return columns;
	}
	
	public int getID() {
		return id;
	}
	public void setID(int id) {
		this.id = id;
	}
	public String getSurname1() {
		return surname1;
	}
	public void setSurname1(String surname1) {
		this.surname1 = Constants.shortString(surname1);
	}
	public String getSurname2() {
		return surname2;
	}
	public void setSurname2(String surname2) {
		this.surname2 = Constants.shortString(surname2);
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = Constants.shortString(name);
	}
	public String getEmail() {
		return email;
	}
	public void setEmail(String email) {
		this.email = Constants.shortString(email);
	}
	public String getSmsphone() {
		return smsphone;
	}
	public void setSmsphone(String smsphone) {
		this.smsphone = Constants.shortString(smsphone);
	}

	@Override
	public String toString() {
		return "Student [email=" + email + ", id=" + id + ", name=" + name
				+ ", surname1=" + surname1 + ", surname2=" + surname2 + "]";
	}
	
	/**
	 * Se añade él mismo a la base de datos
	 * @return el id del Student insertado
	 */
	public int add() {
		return storage.addStudent(this);
	}

	/**
	 * Se elimina él mismo de la base de datos
	 */
	public void delete() {
		storage.deleteStudent(this);
	}

	/**
	 * Se modifica él mismo en la base de datos
	 * @return el código de retorno
	 */
	public int update() {
		return storage.updateStudent(this);
	}
	
	
}
