/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;

public class Teacher implements Entity{
	private int id;
	private String login = null;
	private String password = null;
	private String fullname;
	private String email;
	private String comment;
	private boolean admin;
	
	/**
	 * Constructor de los nuevos Teacher. Al añadir un Teacher desde la aplicación,
	 * se usará este constructor.
	 * @param id
	 * @param login
	 * @param password Es el hash del password
	 * @param fullname
	 * @param email
	 * @param comment
	 * @param admin
	 */
	public Teacher(int id, String login, String password, String fullname, String email, String comment, boolean admin) {
		this.id = id;
		
		if (login.equals("")) {
			Logger.log("Teacher.Teacher(" + id + " ," + login + " ," + password + " ," + fullname + " ," + email + " ," + comment + " ," + admin + ")");
			Logger.log("No se puede crear un profesor con el login '' !!!!");
		} else {
			setLogin(login);
		}
		
		if (password.equals("")) {
			Logger.log("Teacher.Teacher(" + id + " ," + login + " ," + password + " ," + fullname + " ," + email + " ," + comment + " ," + admin + ")");
			Logger.log("No se puede crear un profesor con el password '' !!!!");
		} else {
			setPassword(password);
		}
		
		setFullname(fullname);
		setEmail(email);
		setComment(comment);
		
		this.admin = admin;
	}
	
	/**
	 * Constructor de los Teacher importados. Al importar un Teacher desde un csv,
	 * se usará este constructor.
	 * @param login
	 * @param passwordTextPlain password en texto claro, se calcula el hash y se guarda en el objeto
	 * @param fullname
	 * @param email
	 */
	public Teacher(String login, String passwordTextPlain, String fullname, String email){
		this(0, login, AttendancemanagerApplication.storage.getPasswordHash(passwordTextPlain), fullname, email, null, false);
		Logger.log("Nuevo profesor: " + this);
	}
	
	/**
	 * Método estático que devuelve un array con las cabeceras del csv
	 * @return devuelve un array con las cadenas de las cabeceras que deberían tener los csv de Teacher
	 */
	public static String[] getBasicProperties() {
		String columns[] =  {"login", "password", "fullname", "email"};
		return columns;
	}
	
	public int getID() {
		return id;
	}
	
	public void setID(int id) {
		this.id = id;
	}
	
	public String getLogin() {
		return login;
	}
	
	public void setLogin(String login) {
		this.login = Constants.shortString(login);
	}
	
	public String getPassword() {
		return password;
	}
	
	public void setPassword(String password) {
		this.password = Constants.shortString(password).toLowerCase();
	}
	
	public String getFullname() {
		return fullname;
	}

	public void setFullname(String fullname) {
		this.fullname = Constants.shortString(fullname);
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = Constants.longString(email);
	}
	
	public String getComment() {
		return comment;
	}
	
	public void setComment(String comment) {
		this.comment = Constants.shortString(comment);
	}

	public boolean getAdmin() {
		return admin;
	}

	public void setAdmin(boolean admin) {
		this.admin = admin;
	}
	
	/**
	 * Se añade él mismo a la base de datos
	 * @return el id del Teacher insertado
	 */
	public int add() {
		return storage.addTeacher(this);
	}
	
	/**
	 * Se elimina él mismo de la base de datos
	 */
	public void delete() {
		storage.deleteTeacher(this.id);
	}
	
	/**
	 * Se modifica él mismo en la base de datos
	 * @return el código de retorno
	 */
	public int update() {
		storage.updateTeacher(this);
		return 0; //PROVISIONAL
	}

	@Override
	public String toString() {
		return "Teacher [id=" + id + ", login=" + login + ", password="
				+ password + ", fullname=" + fullname + ", email=" + email
				+ ", comment=" + comment + ", admin=" + admin + "]";
	}

}
