package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.sql.Time;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.Classblock;
import org.iespuigcastellar.attendancemanager.core.Constants;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Subject;
import org.iespuigcastellar.attendancemanager.core.Teacher;
import org.iespuigcastellar.attendancemanager.core.Term;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.AbstractSelect;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.DefaultFieldFactory;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.NativeSelect;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Upload;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Window.Notification;

public class ClassblockManagerLayout extends VerticalLayout{
	private AttendancemanagerApplication app;
	
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	private Button importButton;
	
	private Table classblocksTable;
	
	private NewClassblockWindow newClassblockWindow;
	private EditClassblockWindow editClassblockWindow;
	private DeleteClassblockWindow deleteClassblockWindow;
	/*private ImportClassblockWindow importClassblockWindow;*/
	
	private void initLayout() {
		newButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_NEWBUTTON_CAPTION"));
		newButton.setIcon(new ThemeResource("img/clock_add.png"));
		newButton.setStyleName("default");
		
	
		editButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_EDITBUTTON_CAPTION"));
		editButton.setIcon(new ThemeResource("img/clock_edit.png"));
		editButton.setEnabled(false);
		editButton.setStyleName("default");
		
		deleteButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_DELETEBUTTON_CAPTION"));
		deleteButton.setIcon(new ThemeResource("img/clock_delete.png"));
		deleteButton.setEnabled(false);
		deleteButton.setStyleName("default");
		
		importButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_IMPORTBUTTON_CAPTION"));
		importButton.setIcon(new ThemeResource("img/clock_go.png"));
		importButton.setStyleName("default");
		//NOT IN USE
		importButton.setVisible(false);
		
		CssLayout toolbarclassblock = new CssLayout();
		toolbarclassblock.addComponent(newButton);
		toolbarclassblock.addComponent(editButton);
		toolbarclassblock.addComponent(deleteButton);
		toolbarclassblock.addComponent(importButton);
		toolbarclassblock.setStyleName("toolbar");
		toolbarclassblock.setWidth("100%");
		
		classblocksTable = new Table();
		classblocksTable.setImmediate(true);
		classblocksTable.setColumnReorderingAllowed(true);
		classblocksTable.setColumnCollapsingAllowed(true);
		classblocksTable.setStyleName("striped");
		classblocksTable.setSizeFull();
				

		Panel subjectsPanel = new Panel(app.locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKPANEL_CAPTION"));
		subjectsPanel.addComponent(classblocksTable);
		subjectsPanel.setSizeFull();
		subjectsPanel.setStyleName("bubble");
		subjectsPanel.setIcon(new ThemeResource("img/clock.png"));
		subjectsPanel.getContent().setSizeFull();
			
		
		VerticalLayout vl =  (VerticalLayout) subjectsPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(classblocksTable, 1.0f);
		vl.setSpacing(false);
		
		addComponent(toolbarclassblock);
		addComponent(subjectsPanel);
		
		setSizeFull();
		setMargin(false, true, false, true);
		
		setExpandRatio(subjectsPanel, 1.0f);
		setSpacing(true);
	}
	public void attach() {

		app = (AttendancemanagerApplication)getApplication();
		initLayout();
		classblocksTable.addContainerProperty(DatabaseStorage.TEACHER_IDTEACHER_PROPERTY, Integer.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.SUBJECT_IDSUBJECT_PROPERTY, Integer.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.GROUP_IDGROUP_PROPERTY, Integer.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.TERM_IDTERM_PROPERTY, Integer.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.TEACHERNAME_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.TERMNAME_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.GROUPNAME_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.SUBJECTNAME_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.START_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.END_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.DAYOFWEEKSTRING_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.DAYOFWEEK_PROPERTY, Integer.class, "");
		String visiblecolumns[]={
				DatabaseStorage.TEACHERNAME_PROPERTY,
				DatabaseStorage.TERMNAME_PROPERTY,
				DatabaseStorage.GROUPNAME_PROPERTY,
				DatabaseStorage.SUBJECTNAME_PROPERTY,
				DatabaseStorage.START_PROPERTY,
				DatabaseStorage.END_PROPERTY,
				DatabaseStorage.DAYOFWEEKSTRING_PROPERTY
			};
		
		
		classblocksTable.setVisibleColumns(visiblecolumns);
		classblocksTable.setColumnHeaders(new String[] {
				app.locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_TEACHERNAME"),
				app.locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_TERMNAME"),
				app.locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_GROUPNAME"),
				app.locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_SUBJECTNAME"),
				app.locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_STARTCOLUMN"),
				app.locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_ENDCOLUMN"),
				app.locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_DAYOFWEEK")
				});
		classblocksTable.setSelectable(true);
		classblocksTable.setMultiSelect(false);
		classblocksTable.setImmediate(true);
		classblocksTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				Object value = classblocksTable.getValue();
				if (value!=null){
					editButton.setEnabled(true);
					deleteButton.setEnabled(true);
				} else {
					editButton.setEnabled(false);
					deleteButton.setEnabled(false);
					
				}
			}
		});
	    newClassblockWindow = new NewClassblockWindow();
	    newClassblockWindow.setCaption(app.locale.getString("CLASSBLOCKMANAGERLAYOUT_NEWCLASSBLOCKWINDOW_CAPTION"));
		newButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(newClassblockWindow.getParent() != null) {
					// Window already open
				} else {
					// refresh Combos (Teachers,Terms.....)
					newClassblockWindow.refreshCombos();						
					getWindow().addWindow(newClassblockWindow);
				}
			}
		});
		editClassblockWindow = new EditClassblockWindow();
		editClassblockWindow.setCaption(app.locale.getString("CLASSBLOCKMANAGERLAYOUT_EDITCLASSBLOCKWINDOW_CAPTION"));
		editButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				editClassblockWindow.refreshCombos();
				Integer idItem=(Integer) classblocksTable.getValue();
				Item item=classblocksTable.getItem(idItem);
				Integer idClassblock=(Integer) classblocksTable.getValue();
				Integer dayofweek=(Integer) item.getItemProperty(DatabaseStorage.DAYOFWEEK_PROPERTY).getValue();
				Integer idteacher=(Integer) item.getItemProperty(DatabaseStorage.TEACHER_IDTEACHER_PROPERTY).getValue();
				Integer idgroup=(Integer) item.getItemProperty(DatabaseStorage.GROUP_IDGROUP_PROPERTY).getValue();
				Integer idsubject=(Integer) item.getItemProperty(DatabaseStorage.SUBJECT_IDSUBJECT_PROPERTY).getValue();
				Integer idterm=(Integer) item.getItemProperty(DatabaseStorage.TERM_IDTERM_PROPERTY).getValue();
				SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
				Time start=	Time.valueOf((String) item.getItemProperty(DatabaseStorage.START_PROPERTY).getValue());
				Time end=	Time.valueOf((String) item.getItemProperty(DatabaseStorage.END_PROPERTY).getValue());
				
				Classblock classblock = new Classblock(idClassblock,idterm,idteacher, idsubject, idgroup, dayofweek, start, end);
				editClassblockWindow.setClassblock(classblock);
				getWindow().addWindow(editClassblockWindow);
			}
		});	
		deleteClassblockWindow = new DeleteClassblockWindow();
		deleteClassblockWindow.setCaption(app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DELETECLASSBLOCKWINDOW_CAPTION"));
		deleteButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				Integer idItem=(Integer) classblocksTable.getValue();
				Item item=classblocksTable.getItem(idItem);
				Integer idClassblock=(Integer) classblocksTable.getValue();
				deleteClassblockWindow.setIdClassblock(idClassblock);
				getWindow().addWindow(deleteClassblockWindow);
			}
		});
	/*	
	 * NOT IN USE
	 * 
	 * importButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				getWindow().addWindow(importClassblockWindow);
			}
		});*/
	}
	private boolean isValid (){
		
		//IF combo teachers is empty or the rest you can not add new classblock
		
		if (newClassblockWindow.combos.isEmptyGroups() ||
			newClassblockWindow.combos.isEmptyTeachers() ||
			newClassblockWindow.combos.isEmptyTerms() ||
			newClassblockWindow.combos.isEmptySubjects()){
			
			return false;
		}
		 return true;		
	}
	public void refreshLayout() {
		app.storage.refreshTable(app.storage.getClassblocks(), classblocksTable);
		newClassblockWindow.refreshCombos();
		editClassblockWindow.refreshCombos();
		if (! isValid()){ 
			newButton.setEnabled(false);
		}else{
			newButton.setEnabled(true);
		}
	}	
	private class NewClassblockWindow extends Window {
		
		public final ClassBlockLayout combos = new ClassBlockLayout();
		
		public NewClassblockWindow() {
			
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {		
				@Override
			public void buttonClick(ClickEvent event) {
					Classblock classblock = new Classblock(
								0,
								combos.getSelectTerm(),
								combos.getSelectedTeacher(),
								combos.getSelectedSubject(),
								combos.getSelectedGroup(),
								combos.getSelectedDayofweek(),
								combos.getTimeStart(),
								combos.getTimeEnd()
						);
					Integer newclassblock =app.storage.addClassblock(classblock);
					getParent().getWindow().showNotification(
							app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
	                    	app.locale.getString("CLASSBLOCKMANAGERLAYOUT_TOOLTIP_CLASSBLOCKADDED"),
                            Notification.TYPE_TRAY_NOTIFICATION);
					refreshLayout();
					setValuesDefault();
					classblocksTable.setValue(newclassblock);
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
					setValuesDefault();
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
			VerticalLayout layouts= new VerticalLayout();
			layouts.setSpacing(true);
			layouts.addComponent(combos);
			layouts.addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		
    		 // We want the window to has automatic size !!!
    		layout.addComponent(layouts);
    		layout.setSizeUndefined();
    		setModal(true);
    		setResizable(false);

		}
		private void setValuesDefault(){
			
		}
		private void refreshCombos(){
			combos.loadGroups();
			combos.loadTeachers();
			combos.loadTerms();
			combos.loadSubjects();
			combos.loadDaysofWeek();
			combos.loadHours();
			combos.loadMinutes();
		}
	}
	private class EditClassblockWindow extends Window {
		
		public final ClassBlockLayout combos = new ClassBlockLayout();
		private Classblock classblock;
		public EditClassblockWindow() {
			
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {		
				@Override
			public void buttonClick(ClickEvent event) {
					classblock.setIdteacher(combos.getSelectedTeacher());
					classblock.setIdterm(combos.getSelectTerm());
					classblock.setIDGroup(combos.getSelectedGroup());
					classblock.setIDSubject(combos.getSelectedSubject());
					classblock.setDay_of_week(combos.getSelectedDayofweek());
					classblock.setStart(combos.getTimeStart());
					classblock.setEnd(combos.getTimeEnd());
					if (app.storage.updateClassblock(classblock)!=0) {
						getParent().getWindow().showNotification(
								app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
	                    		app.locale.getString("CLASSBLOCKMANAGERLAYOUT_TOOLTIP_CLASSBLOCKEDITED"),
	                            Notification.TYPE_TRAY_NOTIFICATION);
					};
					refreshLayout();
					classblocksTable.setValue(classblock.getID());
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
				
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
			VerticalLayout layouts= new VerticalLayout();
			layouts.setSpacing(true);
			layouts.addComponent(combos);
			layouts.addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		
    		 // We want the window to has automatic size !!!
    		layout.addComponent(layouts);
    		layout.setSizeUndefined();
    		setModal(true);
    		setResizable(false);

		}
		private void refreshCombos(){
			combos.loadGroups();
			combos.loadTeachers();
			combos.loadTerms();
			combos.loadSubjects();
			combos.loadDaysofWeek();
			combos.loadHours();
			combos.loadMinutes();
		}
		private void setClassblock(Classblock classblock){
			combos.setTeacher(classblock.getIdteacher());
			combos.setTerm(classblock.getIdterm());
			combos.setGroup(classblock.getIdgroup());
			combos.setSubject(classblock.getIDSubject());
			combos.setDayofweek(classblock.getDay_of_week());
			combos.setHourStart(classblock.getStart());
			combos.setHourEnd(classblock.getEnd());
			this.classblock=classblock;
		}
	}	
	private class DeleteClassblockWindow extends Window{
		
		private Subject subject;
		private Label messageLabel;
		private Integer idClassblock=0;
		public DeleteClassblockWindow() {
			
			Embedded warningEmbedded = new Embedded(null,new ThemeResource("img/Warning_icon.png"));
			 
			messageLabel = new Label();
			String messageString = app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DELETEWINDOW_MESSAGE");
			
			messageLabel.setValue(messageString);
			messageLabel.setContentMode(Label.CONTENT_XHTML);
			
			HorizontalLayout warningLayout = new HorizontalLayout();
			warningLayout.setSpacing(true);
			warningLayout.addComponent(warningEmbedded);
			warningLayout.addComponent(messageLabel);
			warningLayout.setSizeFull();
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					app.storage.deleteClassblock(idClassblock);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("CLASSBLOCKMANAGERLAYOUT_TOOLTIP_CLASSBLOCKDELETED"),
                            Notification.TYPE_TRAY_NOTIFICATION);
                    refreshLayout();
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			hl.setSizeFull();
			addComponent(warningLayout);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}

		private void setIdClassblock(Integer idClassblock){
			   this.idClassblock=idClassblock;
		}
	}
	public class  ClassBlockLayout extends VerticalLayout{
		private NativeSelect dayofweek;
		private NativeSelect hourStart;
		private NativeSelect minuteStart;
		private NativeSelect hourEnd;
		private NativeSelect minuteEnd;
		private ComboBox teachers;
		private ComboBox subjects;
		private ComboBox groups;
		private ComboBox terms;
		
		public ClassBlockLayout(){
			setMargin(true);
			GridLayout gl = new GridLayout();
			gl.setRows(8);
			gl.setColumns(10);
			
			HorizontalLayout starttimeLayout = new HorizontalLayout();
			HorizontalLayout endtimeLayout = new HorizontalLayout();
			Label labelStart=new Label(app.locale.getString("CLASSBLOCKMANAGERLAYOUT_LABELSTART_CAPTION"));
			Label labelEnd=new Label(app.locale.getString("CLASSBLOCKMANAGERLAYOUT_LABELEND_CAPTION"));
			
			Label pointsLabel = new Label("<h3>:</h3>",Label.CONTENT_XML);
			Label pointsLabel2 = new Label("<h3>:</h3>",Label.CONTENT_XML);
			
			hourStart=new NativeSelect();
			hourStart.setNullSelectionAllowed(false);
			minuteStart=new NativeSelect();
			minuteStart.setNullSelectionAllowed(false);
			dayofweek=new NativeSelect(app.locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_DAYOFWEEK"));
			dayofweek.setImmediate(true);
			dayofweek.setNullSelectionAllowed(false);
			dayofweek.setInvalidAllowed(false);
			dayofweek.setReadThrough(true);
			dayofweek.setSizeFull();
			starttimeLayout.addComponent(hourStart);
			starttimeLayout.addComponent(pointsLabel);
			starttimeLayout.setComponentAlignment(pointsLabel, Alignment.BOTTOM_CENTER);
			starttimeLayout.addComponent(minuteStart);
			
			hourEnd=new NativeSelect();
			hourEnd.setNullSelectionAllowed(false);
			minuteEnd=new NativeSelect();
			minuteEnd.setNullSelectionAllowed(false);
			
			endtimeLayout.addComponent(hourEnd);
			endtimeLayout.addComponent(pointsLabel2);
			endtimeLayout.setComponentAlignment(pointsLabel2, Alignment.BOTTOM_CENTER);
			endtimeLayout.addComponent(minuteEnd);

			teachers=new ComboBox(app.locale.getString("TEACHERSMANAGERLAYOUT_TEACHERSPANEL_CAPTION"));
			teachers.setSizeFull();
			teachers.setImmediate(true);
			teachers.setRequired(true);
			teachers.setInvalidAllowed(false);
			teachers.setNullSelectionAllowed(false);
			teachers.setFilteringMode(ComboBox.FILTERINGMODE_STARTSWITH);
			teachers.setReadThrough(true);
			
			
			subjects=new ComboBox(app.locale.getString("SUBJECTSMANAGERLAYOUT_SUBJECTSPANEL_CAPTION"));
			subjects.setSizeFull();
			subjects.setImmediate(true);
			subjects.setRequired(true);
			subjects.setInvalidAllowed(false);
			subjects.setNullSelectionAllowed(false);
			subjects.setFilteringMode(ComboBox.FILTERINGMODE_STARTSWITH);
			subjects.setReadThrough(true);
			
			groups=new ComboBox(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSPANEL_CAPTION"));
			groups.setSizeFull();
			groups.setNullSelectionAllowed(false);
			groups.setImmediate(true);
			groups.setRequired(true);
			groups.setInvalidAllowed(false);
			groups.setFilteringMode(ComboBox.FILTERINGMODE_STARTSWITH);
			groups.setReadThrough(true);	
	
			terms=new ComboBox(app.locale.getString("TERMMANAGERLAYOUT_TERMSPANEL_CAPTION"));
			terms.setSizeFull();
			terms.setNullSelectionAllowed(false);
			terms.setImmediate(true);
			terms.setRequired(true);
			terms.setFilteringMode(ComboBox.FILTERINGMODE_STARTSWITH);
			terms.setReadThrough(true);	
	
			
			gl.addComponent(dayofweek,         0,0,9,0);
			gl.addComponent(labelStart,        0,1,3,1);
			gl.addComponent(starttimeLayout,   4,1,9,1);
			gl.addComponent(labelEnd,          0,2,3,2);
			gl.addComponent(endtimeLayout,     4,2,9,2);

			gl.addComponent(teachers,          0,4,9,4);
			gl.addComponent(subjects,          0,5,9,5);
			gl.addComponent(groups,            0,6,9,6);
			gl.addComponent(terms,             0,7,9,7);

			gl.setComponentAlignment(labelStart, Alignment.MIDDLE_CENTER);
			gl.setComponentAlignment(labelEnd, Alignment.MIDDLE_CENTER);
			gl.setSpacing(true);
			addComponent(gl);
		}

		public Boolean isEmptyTeachers(){
			if (teachers.size()==0){return true;}
			return false;
		}
		public Boolean isEmptyTerms(){
			if (terms.size()==0){return true;}
			return false;
		}
		public Boolean isEmptySubjects(){
			if (subjects.size()==0){return true;}
			return false;
		}
		public Boolean isEmptyGroups(){
			if (groups.size()==0){return true;}
			return false;
		}
		private void loadTeachers(){
			//Load Combo Teachers
			Integer firstItem=null;
			Collection collection = app.storage.allTeachers();
			IndexedContainer ict=new IndexedContainer();
			ict.addContainerProperty("fullname", String.class, null);
			Iterator<Teacher> it =collection.iterator();
			Item itemTeacher;
			while (it.hasNext()){
				Teacher teacher=it.next();
				if (firstItem==null){firstItem=teacher.getID();}
				itemTeacher=ict.addItem(teacher.getID());
				itemTeacher.getItemProperty("fullname").setValue(teacher.getFullname());
			}
			teachers.setContainerDataSource(ict);
			teachers.setItemCaptionPropertyId("fullname");
			teachers.setValue(firstItem);
		}
		private void loadTerms(){
			Integer firstItem=null;
			Collection collection = app.storage.allTerms();
			IndexedContainer ic=new IndexedContainer();
			ic.addContainerProperty("nameterm", String.class, null);
			Iterator<Term> it =collection.iterator();
			Item itemTerm;
			while (it.hasNext()){
				Term term=it.next();
				if (firstItem==null){firstItem= term.getIdTerm();}
				itemTerm=ic.addItem(term.getIdTerm());
				itemTerm.getItemProperty("nameterm").setValue(term.getName());
			}
			terms.setContainerDataSource(ic);
			terms.setItemCaptionPropertyId("nameterm");
			terms.setValue(firstItem);
		}
		private void loadSubjects(){
			Collection collection = app.storage.allSubjects();
			Integer firstItem=null;
			IndexedContainer ic=new IndexedContainer();
			ic.addContainerProperty("name", String.class, null);
			Iterator<Subject> it =collection.iterator();
			Item itemSubject;
			while (it.hasNext()){
				Subject subject=it.next();
				if (firstItem==null){firstItem=subject.getID();};
				itemSubject=ic.addItem(subject.getID());
				itemSubject.getItemProperty("name").setValue(subject.getName());
			}
			subjects.setContainerDataSource(ic);
			subjects.setItemCaptionPropertyId("name");
			subjects.setValue(firstItem);
		}
		private void loadDaysofWeek(){
			String days[]={app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_SUNDAY"),
					app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_MONDAY"),
					app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_TUESDAY"),
					app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_WEDNESDAY"),
					app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_THURSDAY"),
					app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_FRIDAY"),
					app.locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_SATURDAY")};
			IndexedContainer icdaysofweek=new IndexedContainer();
			icdaysofweek.addContainerProperty("dayofweek", String.class, null);
			Item dayofweekItem;
			
			for (int i=1;i<8;i++){
				dayofweekItem=icdaysofweek.addItem(i);
				dayofweekItem.getItemProperty("dayofweek").setValue(days[i-1]);
			}
			dayofweek.setContainerDataSource(icdaysofweek);
			dayofweek.setItemCaptionPropertyId("dayofweek");
			dayofweek.setValue(1);
		}
		private void loadGroups(){
			Collection collection = app.storage.allGroups();
			Integer firstItem=null;
			IndexedContainer ic=new IndexedContainer();
			ic.addContainerProperty("namegroup", String.class, null);
			Iterator<Group> it =collection.iterator();
			Item itemGroup;
			while (it.hasNext()){
				Group group=it.next();
				if (firstItem==null){firstItem=group.getId();};
				itemGroup=ic.addItem(group.getId());
				itemGroup.getItemProperty("namegroup").setValue(group.getName());
			}
			groups.setContainerDataSource(ic);
			groups.setItemCaptionPropertyId("namegroup");
			groups.setValue(firstItem);
		}
		public void setHourStart(Time hour){
			//Combo hours
			String hourString=String.valueOf(hour.getHours());
			if (hourString.length()==1){hourString="0"+hourString;}
			hourStart.setValue(hourString);
			
			//Combo minutes
			String minuteString=String.valueOf(hour.getMinutes());
			if (minuteString.length()==1){minuteString="0"+minuteString;}
			minuteStart.setValue(minuteString);
		}
		public void setHourEnd(Time hour){
			//Combo hours
			String hourString=String.valueOf(hour.getHours());
			if (hourString.length()==1){hourString="0"+hourString;}
			hourEnd.setValue(hourString);
			
			//Combo minutes
			String minuteString=String.valueOf(hour.getMinutes());
			if (minuteString.length()==1){minuteString="0"+minuteString;}
			minuteEnd.setValue(minuteString);
		}
		public void setTeacher(int idteacher){
			teachers.setValue(idteacher);
		}
		public void setTerm(int idterm){
			terms.setValue(idterm);
		}
		public void setGroup(int idgroup){
			groups.setValue(idgroup);
		}
		public void setSubject(int idsubject){
			subjects.setValue(idsubject);
		}
		public void setDayofweek(Integer idDay){	
			dayofweek.setValue(idDay);
		}
		public Integer getSelectedTeacher(){
			Integer result =(Integer) teachers.getValue();
			return result;
		}
		public Integer getSelectedDayofweek(){
			Integer result =(Integer) dayofweek.getValue();
			return result;
		}
		public Integer getSelectTerm(){
			Integer result =(Integer) terms.getValue();
			return result;
		}
		public Integer getSelectedGroup(){
			Integer result =(Integer) groups.getValue();
			return result;
		}
		public Integer getSelectedSubject(){
			Integer result =(Integer) subjects.getValue();
			return result;
		}
		private void loadHours(){
			for (int i=0;i<24;i++){
				String timeString= String.valueOf(i);
				if (timeString.length()==1){ timeString="0"+timeString;}
				hourStart.addItem(timeString);
				hourEnd.addItem(timeString);
			}
			hourStart.setValue("00");
			hourEnd.setValue("00");
		}
		private void loadMinutes(){
			
			for (int i=0;i<60;i++){
				String timeString= String.valueOf(i);
				if (timeString.length()==1){ timeString="0"+timeString;}
				minuteStart.addItem(timeString);
				minuteEnd.addItem(timeString);
			}
			minuteStart.setValue("00");
			minuteEnd.setValue("00");
		}
		
		public Time getTimeStart(){
			String hour=hourStart.getValue() + ":" + minuteStart.getValue()+ ":00";
			Time result = Time.valueOf(hour);
			return result;
		}
		public Time getTimeEnd(){
			String hour=hourEnd.getValue() + ":" + minuteEnd.getValue()+ ":00";

			Time result = Time.valueOf(hour);

			return result;
		} 
		

	}


/*	
 * 
 * NOT IN USE
 * 
 * private class ImportClassblockWindow extends Window {

		private Upload upload=new Upload("Select file",null);
		public ImportClassblockWindow() {
			addComponent(upload);
			
			Button okButton = new Button("Ok");
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					refreshClassblocks();
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button("Cancel");
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
	}*/	
}
