  /* AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

/*
 *	TermManagerLayout designed by Xavier Junyent <xavierjunyent@yahoo.es>
 *
 *	TermManagerLayout allows group administration.
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CSVUploader;
import org.iespuigcastellar.attendancemanager.core.Constants;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Logger;
import org.iespuigcastellar.attendancemanager.core.Term;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Button;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.DateField;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.Form;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Upload;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Upload.FailedEvent;
import com.vaadin.ui.Upload.SucceededEvent;
import com.vaadin.ui.Window.Notification;


@SuppressWarnings("serial")
public class TermManagerLayout extends VerticalLayout {
	
	private AttendancemanagerApplication app;
	
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	private Button importButton;
	
	private Table termsTable;

	
	private NewTermWindow newTermWindow;
	private EditTermWindow editTermWindow;
	private DeleteTermWindow deleteTermWindow;
	private ImportTermWindow importTermWindow;

	
	private void initLayout() {
		
		newButton = new Button();
		newButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_NEWBUTTON_CAPTION"));
		newButton.setIcon(new ThemeResource("img/date_add.png"));
		newButton.setStyleName("default");
		
	
		editButton = new Button();
		editButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_EDITBUTTON_CAPTION"));
		editButton.setIcon(new ThemeResource("img/date_edit.png"));
		editButton.setEnabled(false);
		editButton.setStyleName("default");
		
		deleteButton = new Button();
		deleteButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_DELETEBUTTON_CAPTION"));
		deleteButton.setIcon(new ThemeResource("img/date_delete.png"));
		deleteButton.setEnabled(false);
		deleteButton.setStyleName("default");
		
		importButton = new Button();
		importButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_IMPORTBUTTON_CAPTION"));
		importButton.setIcon(new ThemeResource("img/date_go.png"));
		importButton.setStyleName("default");
		
		CssLayout toolbarterms = new CssLayout();
		toolbarterms.addComponent(newButton);
		toolbarterms.addComponent(editButton);
		toolbarterms.addComponent(deleteButton);
		toolbarterms.addComponent(importButton);
		toolbarterms.setStyleName("toolbar");
		toolbarterms.setWidth("100%");
		
		termsTable = new Table();
		termsTable.setImmediate(true);
		termsTable.setColumnReorderingAllowed(true);
		termsTable.setColumnCollapsingAllowed(true);
		termsTable.setStyleName("striped");
		termsTable.setSizeFull();
				
		Panel termsPanel = new Panel();
		termsPanel.setCaption(app.locale.getString("TERMMANAGERLAYOUT_TERMSPANEL_CAPTION"));
		termsPanel.addComponent(termsTable);
		termsPanel.setSizeFull();
		termsPanel.setStyleName("bubble");
		termsPanel.setIcon(new ThemeResource("img/date.png"));
		termsPanel.getContent().setSizeFull();
			
		
		VerticalLayout vl =  (VerticalLayout) termsPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(termsTable, 1.0f);
		vl.setSpacing(false);
		
		addComponent(toolbarterms);
		addComponent(termsPanel);
		
		setSizeFull();
		setMargin(false, true, false, true);
		
		setExpandRatio(termsPanel, 1.0f);
		setSpacing(true);
		
	}
	
	public void attach() {
		Logger.log("TermManagerLayout.attach()");
		
		app = (AttendancemanagerApplication)getApplication();
		
		initLayout();
		
		termsTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		termsTable.addContainerProperty(DatabaseStorage.COMMENT_PROPERTY, String.class, "");
		termsTable.addContainerProperty(DatabaseStorage.START_PROPERTY, String.class, "");
		termsTable.addContainerProperty(DatabaseStorage.END_PROPERTY, String.class, "");
		termsTable.setColumnHeaders(new String[] { app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"),app.locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_STARTCOLUMN"),app.locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_ENDCOLUMN")});
		//app.storage.refreshTable(app.storage.getStudentsQueryContainer(), teachersTable);
		String visiblecolumns[]={DatabaseStorage.NAME_PROPERTY,DatabaseStorage.COMMENT_PROPERTY,DatabaseStorage.START_PROPERTY,DatabaseStorage.END_PROPERTY};
		termsTable.setVisibleColumns(visiblecolumns);
		termsTable.setSelectable(true);
		termsTable.setMultiSelect(false);
		termsTable.setImmediate(true);

	
		termsTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				Object value = termsTable.getValue();
				if (value!=null){
					editButton.setEnabled(true);
					deleteButton.setEnabled(true);
				} else {
					editButton.setEnabled(false);
					deleteButton.setEnabled(false);
					
				}
			}
		});

		newTermWindow = new NewTermWindow();
		newTermWindow.setCaption(app.locale.getString("TERMMANAGERLAYOUT_NEWTERMWINDOW_CAPTION"));
		
		editTermWindow = new EditTermWindow();
		editTermWindow.setCaption(app.locale.getString("TERMMANAGERLAYOUT_EDITTERMWINDOW_CAPTION"));
		
		deleteTermWindow= new DeleteTermWindow();
		deleteTermWindow.setCaption(app.locale.getString("TERMMANAGERLAYOUT_DELETETERMWINDOW_CAPTION"));
		
		try {
			importTermWindow= new ImportTermWindow();
		} catch (SecurityException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (NoSuchMethodException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		importTermWindow.setCaption(app.locale.getString("TERMMANAGERLAYOUT_IMPORTTERMWINDOW_CAPTION"));
		
		newButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(newTermWindow.getParent() != null) {
					// Window already open
				} else {
					// open window
					getWindow().addWindow(newTermWindow);
				}
			}
		});
			editButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				Date start=null;
				Date end=null;
				Integer idItem=(Integer) termsTable.getValue();
				Item item=termsTable.getItem(idItem);
				Integer idTerm=(Integer) termsTable.getValue();
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
				
				String name=(String) item.getItemProperty(DatabaseStorage.NAME_PROPERTY).getValue();
				String comment=(String) item.getItemProperty(DatabaseStorage.COMMENT_PROPERTY).getValue();
				try {
					start=sdf.parse((String) item.getItemProperty(DatabaseStorage.START_PROPERTY).getValue());
					end=sdf.parse((String) item.getItemProperty(DatabaseStorage.END_PROPERTY).getValue());
				} catch (ParseException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				Term term = new Term(idTerm,name,comment,start, end);
				editTermWindow.setTerm(term);
				getWindow().addWindow(editTermWindow);
				}
			});
			deleteButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					// TODO Auto-generated method stub
					Date start=null;
					Date end=null;
					Integer idItem=(Integer) termsTable.getValue();
					Item item=termsTable.getItem(idItem);
					Integer idTerm=(Integer) termsTable.getValue();
					SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
					
					String name=(String) item.getItemProperty(DatabaseStorage.NAME_PROPERTY).getValue();
					String comment=(String) item.getItemProperty(DatabaseStorage.COMMENT_PROPERTY).getValue();
					try {
						start=sdf.parse((String) item.getItemProperty(DatabaseStorage.START_PROPERTY).getValue());
						end=sdf.parse((String) item.getItemProperty(DatabaseStorage.END_PROPERTY).getValue());
					} catch (ParseException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
					Term term = new Term(idTerm,name,comment,start, end);
					deleteTermWindow.setTerm(term);
					
					getWindow().addWindow(deleteTermWindow);
				}
			});
			importButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					// TODO Auto-generated method stub
					getWindow().addWindow(importTermWindow);
				}
			});
		refreshTerms();
	}
	public void refreshTerms() {
		app.storage.refreshTable(app.storage.getTerms(), termsTable);
	}

	/*
	 * Inner classes
	 */
	public class TermInfo {
		public String name = "";
		public String comment = "";
		public java.util.Date start=null;
		public java.util.Date end=null;
		public String getName() {
			return name;
		}
		public void setName(String name) {
			this.name = name;
		}
		public String getComment() {
			return comment;
		}
		public void setComment(String comment) {
			this.comment = comment;
		}
		public java.util.Date getStart() {
			return start;
		}
		public void setStart(java.util.Date start) {
			this.start = start;
		}
		public java.util.Date getEnd() {
			return end;
		}
		public void setEnd(java.util.Date end) {
			this.end = end;
		}

		
			
	}

	private class NewTermWindow extends Window {
		
		private final Form termForm = new Form();
		private final TermInfo termInfo = new TermInfo();
		
		public NewTermWindow() {
			
			termForm.setValidationVisible(true);
			
			BeanItem<TermInfo> termBean = new BeanItem<TermInfo>(termInfo);
			termForm.setItemDataSource(termBean,Arrays.asList ( "name","comment","start","end"));
			
			TextField tf = (TextField)termForm.getField("name");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"));
			tf.setRequiredError(app.locale.getString("TERMMANAGERLAYOUT_NEWTERMWINDOW_NAMEREQUIREDERROR"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_NAMETOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)termForm.getField("comment");
			tf.setRequired(false);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_COMMENTTOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));

					
			DateField pd = (DateField)termForm.getField("start");
			pd.setRequired(true);
			pd.setImmediate(true);
			pd.setRequiredError(app.locale.getString("TERMMANAGERLAYOUT_NEWTERMWINDOW_STARTREQUIREDERROR"));
			pd.setCaption(app.locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_STARTCOLUMN"));
			
			
			pd = (DateField)termForm.getField("end");
			pd.setRequired(true);
			pd.setImmediate(true);
			pd.setRequiredError(app.locale.getString("TERMMANAGERLAYOUT_NEWTERMWINDOW_ENDREQUIREDERROR"));
			pd.setCaption(app.locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_ENDCOLUMN"));
	
			termForm.setImmediate(true);
		
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!termForm.isValid()) return;
					//Teacher(int id, String login, String password, String fullname, String email, String comment, boolean admin)
				
				
					Term term = new Term(0, 
							(String)termForm.getField("name").getValue(), 
							(String)termForm.getField("comment").getValue(),
							(java.util.Date)termForm.getField("start").getValue(),(java.util.Date)termForm.getField("end").getValue());
					
					Integer idnewTerm=app.storage.addTerm(term);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("TERMMANAGERLAYOUT_TOOLTIP_TERMADDED")+ " " +(String)termForm.getField("name").getValue(),
                            Notification.TYPE_TRAY_NOTIFICATION);
					setValuesEmpty();
					refreshTerms();
					termsTable.setValue(idnewTerm);
                    close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
					setValuesEmpty();
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			termForm.getFooter().addComponent(hl);
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
    		addComponent(termForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		private void setValuesEmpty(){
			
			termForm.getField("name").setValue("");
			termForm.getField("comment").setValue("");
			termForm.getField("start").setValue(new java.util.Date());
			termForm.getField("end").setValue(new java.util.Date());
		}
	}
    private class EditTermWindow extends Window {
		
		private final Form termForm = new Form();
		private final TermInfo termInfo = new TermInfo();
		private Term term;
		
		public EditTermWindow() {
			
			termForm.setValidationVisible(true);
			
			BeanItem<TermInfo> termBean = new BeanItem<TermInfo>(termInfo);
			termForm.setItemDataSource(termBean,Arrays.asList ( "name","comment","start","end"));
			
			TextField tf = (TextField)termForm.getField("name");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"));
			tf.setRequiredError(app.locale.getString("TERMMANAGERLAYOUT_NEWTERMWINDOW_NAMEREQUIREDERROR"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_NAMETOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
						
			tf = (TextField)termForm.getField("comment");
			tf.setRequired(false);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_COMMENTTOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));

			DateField pd = (DateField)termForm.getField("start");
			pd.setRequired(true);
			pd.setImmediate(true);
			pd.setRequiredError(app.locale.getString("TERMMANAGERLAYOUT_NEWTERMWINDOW_STARTREQUIREDERROR"));
			pd.setCaption(app.locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_STARTCOLUMN"));
			
			pd = (DateField)termForm.getField("end");
			pd.setRequired(true);
			pd.setImmediate(true);
			pd.setRequiredError(app.locale.getString("TERMMANAGERLAYOUT_NEWTERMWINDOW_ENDREQUIREDERROR"));
			pd.setCaption(app.locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_ENDCOLUMN"));
	
			termForm.setImmediate(true);
				
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!termForm.isValid()) return;
					//Teacher(int id, String login, String password, String fullname, String email, String comment, boolean admin)
					Term editedterm = new Term(term.getIdTerm(), 
							(String)termForm.getField("name").getValue(), 
							(String)termForm.getField("comment").getValue(),
							(java.util.Date)termForm.getField("start").getValue(),(java.util.Date)termForm.getField("end").getValue());
					
					app.storage.updateTerm(editedterm);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("TERMMANAGERLAYOUT_TOOLTIP_TERMEDITED")+ " " +(String)termForm.getField("name").getValue(),
                            Notification.TYPE_TRAY_NOTIFICATION);
					setValuesEmpty();
					refreshTerms();
					termsTable.setValue(editedterm.getIdTerm());
                    close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
					setValuesEmpty();
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			termForm.getFooter().addComponent(hl);
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
    		addComponent(termForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		private void setValuesEmpty(){
			termForm.getField("name").setValue("");
			termForm.getField("comment").setValue("");
			termForm.getField("start").setValue(new java.util.Date());
			termForm.getField("end").setValue(new java.util.Date());
		}
		public void setTerm(Term term){
			this.term=term;
			termForm.getField("name").setValue(term.getName());
			termForm.getField("comment").setValue(term.getComment());
			termForm.getField("start").setValue(term.getStart());
			termForm.getField("end").setValue(term.getEnd());
			
		}
	}
	private class DeleteTermWindow extends Window {
		
		private Term term;
		private Label messageLabel;

		public DeleteTermWindow() {
			Embedded warningEmbedded = new Embedded(null,new ThemeResource("img/Warning_icon.png"));
			 
			messageLabel = new Label();
			String messageString = app.locale.getString("TERMMANAGERLAYOUT_DELETEWINDOW_MESSAGE");
			
			messageLabel.setValue(messageString);
			messageLabel.setContentMode(Label.CONTENT_XHTML);
			
			HorizontalLayout warningLayout = new HorizontalLayout();
			warningLayout.setSpacing(true);
			warningLayout.addComponent(warningEmbedded);
			warningLayout.addComponent(messageLabel);
			warningLayout.setSizeFull();
			
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					app.storage.deleteTerm(term.getIdTerm());
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("TERMMANAGERLAYOUT_TOOLTIP_TERMDELETED")+ " " +(String)term.getName(),
                            Notification.TYPE_TRAY_NOTIFICATION);
					refreshTerms();
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			hl.setSizeFull();
			addComponent(warningLayout);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		public void setTerm(Term term){
			this.term=term;
			
		}
	}
	
	/**
	 * Ventana emergente de importación de datos en formato csv.
	 * 
	 * @author apalacios
	 *
	 */
	private class ImportTermWindow extends Window implements Upload.SucceededListener, Upload.FailedListener{
		
		private CSVUploader csvuploader = new CSVUploader(Term.class.getConstructor(String.class, String.class, String.class, String.class), Term.getBasicProperties());
		private Upload upload=new Upload(app.locale.getString("GROUPMANAGERLAYOUT_UPLOAD_CAPTION"),csvuploader);
		
		/**
		 * Constructor de la ventana de importación de csv. Crea un objeto Upload y lo conecta con el CSVUploader.
		 * @throws SecurityException
		 * @throws NoSuchMethodException
		 */
		public ImportTermWindow() throws SecurityException, NoSuchMethodException {
			
			Date d = new Date();
			String date = AttendancemanagerApplication.df.format(d);
			String date2 = AttendancemanagerApplication.df.format(d);
			
			String alert = app.locale.getString("TERMMANAGERLAYOUT_ALERTWINDOW_MESSAGE") + date + "\"</th>" + "<th> \"" + date2 + "\"</th>" + "</tr>" + "</table>";
			
			csvuploader.setAlert(alert);
			
			addComponent(upload);
			upload.setButtonCaption(app.locale.getString("GROUPMANAGERLAYOUT_UPLOADBUTTON_CAPTION"));
			
			upload.addListener((Upload.FinishedListener)csvuploader);
			upload.addListener((Upload.SucceededListener)this);
			upload.addListener((Upload.FailedListener)this);
						
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(cancelButton);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		
		/**
		 * Evento de carga correcta. Muestra una notificación con el número de registros correctos y fallidos.
		 */
		@Override
		public void uploadSucceeded(SucceededEvent event) {
			refreshTerms();
			close();
			System.out.println("Plazos cargados !!!");
			
			String finishMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.successfulRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSOK_MESSAGE")
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.failedRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSNOK_MESSAGE");
					
			Label finishLabel = csvuploader.getLabelFromString(finishMessage);
			AttendancemanagerApplication.adminWindow.showNotification(app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_MESSAGE"),(String) finishLabel.getValue(), Notification.TYPE_TRAY_NOTIFICATION);
		}
		
		/**
		 * Evento de carga fallida. Si la carga falla, muestra una alerta de error.
		 */
		@Override
		public void uploadFailed(FailedEvent event) {
			System.out.println("Ohhhh, no he podido insertar los plazos !!!");
			
			String errorMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADFAILED_ERRORSTRING");
			Label errorLabel = csvuploader.getLabelFromString(errorMessage);
			AttendancemanagerApplication.adminWindow.showNotification("ERROR",(String) errorLabel.getValue(), Notification.TYPE_ERROR_MESSAGE);
		}
	}
}
