/* 
 * Copyright 2009 IT Mill Ltd.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package com.vaadin.terminal;

import java.io.Serializable;
import java.util.Map;

/**
 * Interface implemented by all the classes capable of handling external
 * parameters.
 * 
 * <p>
 * Some terminals can provide external parameters for application. For example
 * GET and POST parameters are passed to application as external parameters on
 * Web Adapter. The parameters can be received at any time during the
 * application lifecycle. All the parameter handlers implementing this interface
 * and registered to {@link com.vaadin.ui.Window} receive all the parameters got
 * from the terminal in the given window.
 * </p>
 * 
 * @author IT Mill Ltd.
 * @version
 * 6.2.3
 * @since 3.0
 */
public interface ParameterHandler extends Serializable {

    /**
     * <p>
     * Handles the given parameters. The parameters are given as inmodifieable
     * name to value map. All parameters names are of type:
     * {@link java.lang.String}. All the parameter values are arrays of strings.
     * </p>
     * 
     * @param parameters
     *            the Inmodifiable name to value[] mapping.
     * 
     */
    public void handleParameters(Map<String, String[]> parameters);

    /**
     * ParameterHandler error event.
     */
    public interface ErrorEvent extends Terminal.ErrorEvent {

        /**
         * Gets the source ParameterHandler.
         * 
         * @return the source Parameter Handler.
         */
        public ParameterHandler getParameterHandler();

    }

}
