/* 
 * Copyright 2009 IT Mill Ltd.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package com.vaadin.terminal;

import java.io.Serializable;

/**
 * Interface for different terminal types.
 * 
 * @author IT Mill Ltd.
 * @version
 * 6.2.3
 * @since 3.0
 */
public interface Terminal extends Serializable {

    /**
     * Gets the name of the default theme.
     * 
     * @return the Name of the terminal window.
     */
    public String getDefaultTheme();

    /**
     * Gets the width of the terminal window in pixels.
     * 
     * @return the Width of the terminal window.
     */
    public int getScreenWidth();

    /**
     * Gets the height of the terminal window in pixels.
     * 
     * @return the Height of the terminal window.
     */
    public int getScreenHeight();

    /**
     * Terminal error event.
     */
    public interface ErrorEvent extends Serializable {

        /**
         * Gets the contained throwable.
         */
        public Throwable getThrowable();

    }

    /**
     * Terminal error listener interface.
     */
    public interface ErrorListener extends Serializable {

        /**
         * Invoked when terminal error occurs.
         * 
         * @param event
         *            the fired event.
         */
        public void terminalError(Terminal.ErrorEvent event);
    }
}
